/**
 *
 */
package com.ferri.core;

import java.io.Serializable;

import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.HibernateTemplate;

/**
 * @author dferri
 *
 */
public class SafeHibernateTemplate extends HibernateTemplate {

	/**
	 *
	 */
	public SafeHibernateTemplate() {
		// do nothing more
	}

	/**
	 * @param sessionFactory
	 * @param allowCreate
	 */
	public SafeHibernateTemplate(SessionFactory sessionFactory, boolean allowCreate) {
		super(sessionFactory, allowCreate);
	}

	/**
	 * @param sessionFactory
	 */
	public SafeHibernateTemplate(SessionFactory sessionFactory) {
		super(sessionFactory);
	}

	@Override
	public Object load(final String entityName, final Serializable id, final LockMode lockMode) throws DataAccessException {

		return execute(new HibernateCallback() {
			public Object doInHibernate(Session session) throws HibernateException {
				Object ret = null;
				if (lockMode != null) {
					ret = session.load(entityName, id, lockMode);
				} else {
					ret = session.load(entityName, id);
				}

				confirmCglibProxy(ret);

				return ret;
			}
		}, true);
	}

	@Override
	@SuppressWarnings("unchecked")
	public Object load(final Class entityClass, final Serializable id, final LockMode lockMode) throws DataAccessException {

		return execute(new HibernateCallback() {
			public Object doInHibernate(Session session) throws HibernateException {
				Object ret = null;
				if (lockMode != null) {
					ret = session.load(entityClass, id, lockMode);
				} else {
					ret = session.load(entityClass, id);
				}

				confirmCglibProxy(ret);

				return ret;
			}

		}, true);
	}

	/**
	 * Verify the validity of proxy with the cglib
	 *
	 * @param ret
	 */
	private void confirmCglibProxy(Object ret) throws HibernateException {
		if (ret != null) {
			Hibernate.initialize(ret);
		}
	}
}
