/**
 * 
 */
package com.ferri.core.vo.hibernate.type;

import java.util.Properties;

import org.hibernate.type.CharBooleanType;
import org.hibernate.usertype.ParameterizedType;
import org.springframework.util.Assert;

/**
 * a Customizable Class for types that map an SQL CHAR(1) to a Java Boolean.<br>
 * 
 * @author dferri
 * @see ParameterizedType for specific configuration
 * 
 */
public class CustomCharBooleanType extends CharBooleanType implements ParameterizedType {

	public static final String NAME_VALUE = "nameValue";
	public static final String TRUE_STRING_VALUE = "trueStringValue";
	public static final String FALSE_STRING_VALUE = "falseStringValue";
	/**
	 * 
	 */
	private static final long serialVersionUID = -1595601704244773904L;
	private String falseStringValue = null;
	private String trueStringValue = null;
	private String nameValue = null;

	/**
	 * @see org.hibernate.type.CharBooleanType#getFalseString()
	 */
	@Override
	protected String getFalseString() {
		return getFalseStringValue();
	}

	/**
	 * @see org.hibernate.type.CharBooleanType#getTrueString()
	 */
	@Override
	protected String getTrueString() {
		return getTrueStringValue();
	}

	/**
	 * @see org.hibernate.type.BooleanType#getName()
	 */
	@Override
	public String getName() {
		return getNameValue();
	}

	public void setParameterValues(Properties parameters) {
		setFalseStringValue(validPropertyValue(parameters, FALSE_STRING_VALUE));
		setTrueStringValue(validPropertyValue(parameters, TRUE_STRING_VALUE));
		setNameValue(validPropertyValue(parameters, NAME_VALUE));
	}

	/**
	 * @param parameters
	 * @param propertyName
	 * @return
	 */
	private String validPropertyValue(Properties parameters, String propertyName) {
		String propertyValue = parameters.getProperty(propertyName);
		Assert.notNull(propertyValue, "the '" + propertyName + "' must not be null");
		return propertyValue;
	}

	/**
	 * @return the falseStringValue
	 */
	public String getFalseStringValue() {
		return falseStringValue;
	}

	/**
	 * @param falseStringValue
	 *            the falseStringValue to set
	 */
	public void setFalseStringValue(String falseStringValue) {
		this.falseStringValue = falseStringValue;
	}

	/**
	 * @return the trueStringValue
	 */
	public String getTrueStringValue() {
		return trueStringValue;
	}

	/**
	 * @param trueStringValue
	 *            the trueStringValue to set
	 */
	public void setTrueStringValue(String trueStringValue) {
		this.trueStringValue = trueStringValue;
	}

	/**
	 * @return the nameValue
	 */
	public String getNameValue() {
		return nameValue;
	}

	/**
	 * @param nameValue
	 *            the nameValue to set
	 */
	public void setNameValue(String nameValue) {
		this.nameValue = nameValue;
	}

}
