package com.ferri.portlet.administrador.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.Controller;

import com.ferri.portlet.utils.Missatge;
import com.ferri.portlet.utils.ValidaCamps;
import com.ferri.service.dto.AdministradorDTO;
import com.ferri.service.dto.GestoriaDTO;
import com.ferri.service.dto.input.AdministradorSrvInDTO;
import com.ferri.service.dto.input.GestoriaSrvInDTO;
import com.ferri.service.dto.output.AdministradorSrvOutDTO;
import com.ferri.service.dto.output.GestoriaSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.GestoriaService;

/**
 * Controlador de les pantalles de llistat.jsp i manteniment.jsp de la entitat Administrador
 * @author dferri
 *
 */
public class ViewAdministradorAction implements Controller{
	
	private static final Log log = LogFactory.getLog(ViewAdministradorAction.class);

	private GestoriaService gestoriaService = null;

	private static final String PAGE_MANT	= "administrador/manteniment";
	private static final String PAGE_LIST	= "administrador/llista";
	
	public void handleActionRequest(ActionRequest request, ActionResponse response) throws Exception {
	}
	
	
	public ModelAndView handleRenderRequest(RenderRequest request, RenderResponse response)	throws Exception {
		ModelAndView modView = null;
		
		String reqCode = request.getParameter("reqCode");
				
		if ((reqCode != null) && (!reqCode.equals(""))) {
			
			if (reqCode.equals("eliminar")){
				if(elimina(request)){
					modView = new ModelAndView(PAGE_LIST);
					defecte(request);
				}else{
					modView = new ModelAndView(PAGE_MANT);
					subElements(request);
				}			
			}else if (reqCode.equals("afegir")) {
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
			
			}else if(reqCode.equals("editar")) {				
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				editar(request);
				
			}else if (reqCode.equals("guardar")) {
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				request.setAttribute("result", guardar(request));
				
			}else if (reqCode.equals("buscar")){
				modView = new ModelAndView(PAGE_LIST);
				buscar(request);
			}
			
			System.out.println("----->" + reqCode);
		
		}else{			
			modView = new ModelAndView(PAGE_LIST);
			defecte(request);
		}
		
		return modView;
	}
	
	/**
	 * Carrega el llistat amb tots els elements Administrador i l'afegeix a la request. 
	 * @param request
	 * @throws CommonServiceException
	 */
	private void defecte(PortletRequest request) throws CommonServiceException{
		//Mostrar todos los elementos por defecto
		AdministradorSrvOutDTO srvOutDTO = this.gestoriaService.getAllAdministrador();
		Collection<AdministradorDTO> lista = srvOutDTO.getAdministradorDTOList();
		
		if (lista != null) {
			for (AdministradorDTO administradorDTO : lista) {
				administradorDTO.setNumClients(this.gestoriaService.countClientsAdministradors(administradorDTO.getNif()));
			}
			request.setAttribute("dtoList", lista);
		}		
	}
	
	/**
	 * Carrega els llistats d'elements secundaris: Categories, i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void subElements(PortletRequest request) throws CommonServiceException{
		GestoriaSrvInDTO gestoriaSrvInDTO = new GestoriaSrvInDTO();
		GestoriaDTO gestoriaDTO = new GestoriaDTO();
		gestoriaDTO.setOrdre("nom");
		gestoriaSrvInDTO.setGestoriaDTO(gestoriaDTO);
		GestoriaSrvOutDTO gestoriaSrvOutDTO = this.gestoriaService.getGestoriasByCriteria(gestoriaSrvInDTO);
		if(gestoriaSrvOutDTO != null){
			Collection<GestoriaDTO> gestoriaDTOs = gestoriaSrvOutDTO.getGestoriaDTOList();
			request.setAttribute("llistaGestoriesDto", gestoriaDTOs);
		}
	}
	
	/**
	 * Elimina una entitat Administrador amb la pk pasada en la request.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private boolean elimina(PortletRequest request) throws Exception {
		log.debug("delete(request)");

		//Long id = Long.valueOf(Long.parseLong(request.getParameter("id")));
		String id = String.valueOf((request.getParameter("id")));

		AdministradorSrvInDTO 	srvInDTO 	= new AdministradorSrvInDTO();
		AdministradorDTO 		dto			= new AdministradorDTO();

		dto.setNif(id);
		srvInDTO.setAdministradorDTO(dto);
		Boolean deleted = false;
		try{
			deleted = this.gestoriaService.deleteAdministrador(srvInDTO);
		}catch(DataIntegrityViolationException e){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.elimina", Missatge.TYPE_ERROR));
			AdministradorSrvOutDTO srvOutDTO = this.gestoriaService.getAdministrador(srvInDTO);
			dto = srvOutDTO.getAdministradorDTO();
			request.setAttribute("dtoObject", dto);
			return false;
		}
		if(deleted){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.elimina", Missatge.TYPE_SUCCES));
		}
		log.debug("deleted-->\t" + deleted);
		return deleted;
	}

	/**
	 * Guarda una entitat Administrador, previa validaci, amb tots els camps del forumlari pasats per request: pot crear-lo de nou si no existeix o actualitzar-lo si esxisteix.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private Boolean guardar(PortletRequest request) throws Exception {
		log.debug("guardar(request)");

		String nif 				= request.getParameter("nif");
		String nom				= request.getParameter("nom");
		String telefon			= request.getParameter("telefon");
		String mail				= request.getParameter("mail");
		String nif_gestoria		= request.getParameter("nif_gestoria");
		
		AdministradorDTO 		dto			= new AdministradorDTO();

		dto.setNif(nif);
		dto.setNom(nom);
		dto.setTelefon(telefon);
		dto.setMail(mail);
		
		GestoriaDTO gestoria = new GestoriaDTO();
		gestoria.setNif(nif_gestoria);
		dto.setGestoria(gestoria);
		
		dto.setDataModif(new Date(System.currentTimeMillis()));

		if(!validaCamps(dto, request)){
			request.setAttribute("dtoObject", dto);
			request.setAttribute("persistObject", Boolean.FALSE);
			return false;
		}
		
		AdministradorSrvInDTO 	srvInDTO 	= new AdministradorSrvInDTO();
		srvInDTO.setAdministradorDTO(dto);		
		AdministradorSrvOutDTO srvOutDTO = this.gestoriaService.getAdministrador(srvInDTO);

		Boolean nou = true;
		if(srvOutDTO != null && srvOutDTO.getAdministradorDTO() != null){
			nou = false;
			AdministradorDTO dto2 = srvOutDTO.getAdministradorDTO();
			dto.setDataAlta(dto2.getDataAlta());
			srvInDTO.setAdministradorDTO(dto);
		}
				
		if(nou){
			dto.setDataAlta(new Date(System.currentTimeMillis()));
			srvInDTO.setAdministradorDTO(dto);
			String id = this.gestoriaService.createAdministrador(srvInDTO);
			//dto.setNif(id);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.alta", Missatge.TYPE_SUCCES));
			log.debug("id newAdministrador--> " + id);
		}else{
			Boolean updated = this.gestoriaService.updateAdministrador(srvInDTO);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica", Missatge.TYPE_SUCCES));
			log.debug("updated Administrador--> " + updated);
		}
		
		request.setAttribute("dtoObject", dto);
		return true;
	}
	
	/**
	 * Carrega una entitat Administrador en request a partir del nif pasat tamb per request.
	 * @param request
	 * @throws Exception
	 */
	private void editar(PortletRequest request) throws Exception {
		log.debug("editar(request)");

		String nif 						= request.getParameter("nif");		
		AdministradorDTO dto			= new AdministradorDTO();
		dto.setNif(nif);
		
		AdministradorSrvInDTO 	srvInDTO 	= new AdministradorSrvInDTO();
		srvInDTO.setAdministradorDTO(dto);
		
		AdministradorSrvOutDTO srvOutDTO = this.gestoriaService.getAdministrador(srvInDTO);
		dto = srvOutDTO.getAdministradorDTO();
		request.setAttribute("dtoObject", dto);
	}
	
	/**
	 * Cerca un llistat d'entitats Administrador que compleixin els criteris i ordenaci passats per formulari request.
	 * El llistat resultant es passat per request.
	 * @param request
	 * @throws Exception
	 */
	private void buscar(PortletRequest request) throws Exception {
		log.debug("buscar(request)");

		String nif = request.getParameter("nif");
		String nom = request.getParameter("nom");
		String ordre = request.getParameter("ordre");
		
		//Provinientes de otra pagina
		String nif_gestoria = request.getParameter("nif_gestoria");
		GestoriaDTO gestoriaDTO = new GestoriaDTO();   
		gestoriaDTO.setNif(nif_gestoria);
		
		AdministradorSrvInDTO 	srvInDTO 	= new AdministradorSrvInDTO();
		AdministradorDTO 		dto			= new AdministradorDTO();

		dto.setNif(nif);
		dto.setNom(nom);
		dto.setGestoria(gestoriaDTO);
		dto.setOrdre(ordre);
		request.setAttribute("dtoObject", dto);
		srvInDTO.setAdministradorDTO(dto);

		AdministradorSrvOutDTO srvOutDTO = this.gestoriaService.getAdministradorsByCriteria(srvInDTO);
		Collection<AdministradorDTO> listaDTO = srvOutDTO.getAdministradorDTOList();
		if(listaDTO == null || listaDTO.isEmpty()){
			request.setAttribute("missatges", Missatge.addMissatge("llistat.info.cerca", Missatge.TYPE_INFO));
		}
		if (listaDTO != null) {
			for (AdministradorDTO administradorDTO : listaDTO) {
				administradorDTO.setNumClients(this.gestoriaService.countClientsAdministradors(administradorDTO.getNif()));
			}
		}
		request.setAttribute("dtoList", listaDTO);		
		
		log.debug("listaDTO --> " + listaDTO);
	}	
	
	/**
	 * Valida els camps del formulari de l'entitat Administrador abans de guardar-los.
	 * Si troba errors son afegits per request.
	 * @param dto AdministradorDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaCamps(AdministradorDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valNull(dto.getNif(), missatges) 											&&
				ValidaCamps.valNull(dto.getNom(), missatges) 											&&
				ValidaCamps.valNull(dto.getTelefon(), missatges)										&&
				ValidaCamps.valNull(dto.getMail(), missatges)											&&
				ValidaCamps.valNull(dto.getGestoria().getNif(), missatges) 								&&
				ValidaCamps.valNIFCIF(dto.getNif(), missatges) 											&&
				ValidaCamps.valMail(dto.getMail(), missatges)){			
			return true;
		}else{
			if(	!ValidaCamps.valNull(dto.getNif(), new ArrayList<Missatge>()) ||
				!ValidaCamps.valNIFCIF(dto.getNif(), new ArrayList<Missatge>())){
				dto.setValidationNif(dto.getNif());
				dto.setNif(null);
			}						
			request.setAttribute("missatges", missatges);
			return false;
		}
	}
	
	/**
	 * @return the gestoriaService
	 */
	public GestoriaService getGestoriaService() {
		return gestoriaService;
	}

	/**
	 * @param gestoriaService the gestoriaService to set
	 */
	public void setGestoriaService(GestoriaService gestoriaService) {
		this.gestoriaService = gestoriaService;
	}
	
}