package com.ferri.portlet.client.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.Controller;

import com.ferri.persistence.bean.impl.ClientVO;
import com.ferri.persistence.bean.impl.TipusClientVO;
import com.ferri.portlet.utils.Missatge;
import com.ferri.portlet.utils.ValidaCamps;
import com.ferri.service.dto.AdministradorDTO;
import com.ferri.service.dto.ClientDTO;
import com.ferri.service.dto.GestoriaDTO;
import com.ferri.service.dto.TipusClientDTO;
import com.ferri.service.dto.input.AdministradorSrvInDTO;
import com.ferri.service.dto.input.ClientSrvInDTO;
import com.ferri.service.dto.input.GestoriaSrvInDTO;
import com.ferri.service.dto.output.AdministradorSrvOutDTO;
import com.ferri.service.dto.output.ClientSrvOutDTO;
import com.ferri.service.dto.output.GestoriaSrvOutDTO;
import com.ferri.service.dto.output.TipusClientSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.ClientService;
import com.ferri.service.interfaces.GestoriaService;

/**
 * Controlador de les pantalles de llistat.jsp i manteniment.jsp de la entitat Client
 * @author dferri
 *
 */
public class ViewClientAction implements Controller { 
	
	private static final Log log = LogFactory.getLog(ViewClientAction.class);

	private ClientService clientService = null;
	private GestoriaService gestoriaService = null;

	private static final String PAGE_MANT	= "client/manteniment";
	private static final String PAGE_LIST	= "client/llista";
	
	public void handleActionRequest(ActionRequest request, ActionResponse response) throws Exception {	
	}
	
	
	public ModelAndView handleRenderRequest(RenderRequest request, RenderResponse response)	throws Exception {
		ModelAndView modView = null;
		
		String reqCode = request.getParameter("reqCode");
				
		if ((reqCode != null) && (!reqCode.equals(""))) {
			
			if (reqCode.equals("eliminar")){
				if(elimina(request)){
					modView = new ModelAndView(PAGE_LIST);
					defecte(request);
				}else{
					modView = new ModelAndView(PAGE_MANT);
					subElements(request);
				}			
			}else if (reqCode.equals("afegir")) {
				modView = new ModelAndView(PAGE_MANT);				
				subElements(request);
			
			}else if(reqCode.equals("editar")) {
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				editar(request);
				
			}else if (reqCode.equals("guardar")) {
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				request.setAttribute("result", guardar(request));
				
			}else if (reqCode.equals("buscar")){
				modView = new ModelAndView(PAGE_LIST);
				buscar(request);
			}
			
			System.out.println("----->" + reqCode);
		
		}else{			
			modView = new ModelAndView(PAGE_LIST);
			defecte(request);
		}
		
		return modView;
	}
	
	/**
	 * Carrega el llistat amb tots els elements Client i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void defecte(PortletRequest request) throws CommonServiceException{
		//Mostrar todos los elementos por defecto
		ClientSrvOutDTO srvOutDTO = this.clientService.getAllClient();
		Collection<ClientDTO> lista = srvOutDTO.getClientDTOList();
		if(lista != null){
			for (ClientDTO clientDTO : lista) {
				clientDTO.setColorSemaforFactura(this.clientService.getSemaforEstatFactures(clientDTO.getNif()));
				clientDTO.setColorSemaforIntervencio(this.clientService.getSemaforEstatIntervencions(clientDTO.getNif()));
			}
		}
		
		if (lista != null) {
			request.setAttribute("dtoList", lista);
		}		
	}
	
	/**
	 * Carrega els llistats d'elements secundaris: TipusClient, Gestories i Administradors per cada gestoria i els afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void subElements(PortletRequest request) throws CommonServiceException{
		GestoriaSrvInDTO gestoriaSrvInDTO = new GestoriaSrvInDTO();
		GestoriaDTO gestoriaDTO = new GestoriaDTO();
		gestoriaDTO.setOrdre("nom");
		gestoriaSrvInDTO.setGestoriaDTO(gestoriaDTO);
		GestoriaSrvOutDTO gestoriaSrvOutDTO = this.gestoriaService.getGestoriasByCriteria(gestoriaSrvInDTO);
		if(gestoriaSrvOutDTO != null){
			Collection<GestoriaDTO> gestoriaDTOs = gestoriaSrvOutDTO.getGestoriaDTOList();
			Collection<GestoriaDTO> gestoriaDefDTOs = new ArrayList<GestoriaDTO>();
			for (GestoriaDTO gestoriaDTO2 : gestoriaDTOs) {
				AdministradorSrvInDTO administradorSrvInDTO = new AdministradorSrvInDTO();
				AdministradorDTO administradorDTO = new AdministradorDTO();
				administradorDTO.setOrdre("nom");
				administradorDTO.setGestoria(gestoriaDTO2);
				administradorSrvInDTO.setAdministradorDTO(administradorDTO);
				AdministradorSrvOutDTO administradorSrvOutDTO = this.gestoriaService.getAdministradorsByCriteria(administradorSrvInDTO);
				if(administradorSrvOutDTO != null){
					Collection<AdministradorDTO> administradorDTOs = administradorSrvOutDTO.getAdministradorDTOList();
					gestoriaDTO2.setAdministradorDTOs(administradorDTOs);
					gestoriaDefDTOs.add(gestoriaDTO2);
				}
			}
			request.setAttribute("llistaGestoriesDto", gestoriaDefDTOs);
		}
		
		//No hace falta ordenar en este caso
		TipusClientSrvOutDTO clientSrvOutDTO = this.clientService.getAllTipusClient();
		if(clientSrvOutDTO != null){
			Collection<TipusClientDTO> tipusClientsDTOs = clientSrvOutDTO.getTipusClientDTOList();
			request.setAttribute("llistaTipusClientDto", tipusClientsDTOs);
		}
		
	}
	
	/**
	 * Elimina una entitat Client amb la pk pasada en la request.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private boolean elimina(PortletRequest request) throws Exception {
		log.debug("delete(request)");

		//Long id = Long.valueOf(Long.parseLong(request.getParameter("id")));
		String id = String.valueOf((request.getParameter("id")));

		ClientSrvInDTO 	srvInDTO 	= new ClientSrvInDTO();
		ClientDTO 		dto			= new ClientDTO();

		dto.setNif(id);
		srvInDTO.setClientDTO(dto);
		Boolean deleted = false;
		try{
			deleted = this.clientService.deleteClient(srvInDTO);
		}catch(DataIntegrityViolationException e){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.elimina", Missatge.TYPE_ERROR));
			ClientSrvOutDTO srvOutDTO = this.clientService.getClient(srvInDTO);
			dto = srvOutDTO.getClientDTO();
			request.setAttribute("dtoObject", dto);
			return false;
		}
		if(deleted){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.elimina", Missatge.TYPE_SUCCES));
		}
		log.debug("deleted-->\t" + deleted);
		return deleted;
	}

	/**
	 * Guarda una entitat Client, previa validaci, amb tots els camps del forumlari pasats per request: pot crear-lo de nou si no existeix o actualitzar-lo si esxisteix.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private Boolean guardar(PortletRequest request) throws Exception {
		log.debug("guardar(request)");

		String nif 				= request.getParameter("nif");
		String nom				= request.getParameter("nom");
		String carrer			= request.getParameter("carrer");
		String numero			= request.getParameter("numero");
		String resta			= request.getParameter("resta");
		String municipi			= request.getParameter("municipi");
		String cp				= request.getParameter("cp");		
		String telefon			= request.getParameter("telefon");
		String mail				= request.getParameter("mail");
		String nif_gestoria		= request.getParameter("nif_gestoria");
		String nif_administrador= request.getParameter("nif_administrador");
		String tipus_client		= request.getParameter("tipus_client");
		
		ClientDTO 		dto			= new ClientDTO();

		dto.setNif(nif);
		dto.setNom(nom);
		dto.setCarrer(carrer);
		dto.setNumero(numero);
		dto.setResta(resta);
		dto.setMunicipi(municipi);
		dto.setCp(cp);
		dto.setTelefon(telefon);
		dto.setMail(mail);
		
		TipusClientDTO tipusClient = new TipusClientDTO();
		tipusClient.setId(tipus_client != null ? Long.parseLong(tipus_client): null);
		dto.setTipusClient(tipusClient);
		
		if(tipusClient != null && tipusClient.getId().intValue() == TipusClientVO.COMUNITAT.intValue()){
			if(nif_gestoria != null && !nif_gestoria.trim().equals("")){
				GestoriaDTO gestoria = new GestoriaDTO();
				gestoria.setNif(nif_gestoria);
				dto.setGestoria(gestoria);
				
				if(nif_administrador != null && !nif_administrador.trim().equals("")){
					AdministradorDTO administrador = new AdministradorDTO();
					administrador.setNif(nif_administrador);
					dto.setAdministrador(administrador);
				}
			}
		}
		
		dto.setDataModif(new Date(System.currentTimeMillis()));

		if(!validaCamps(dto, request)){
			request.setAttribute("dtoObject", dto);
			request.setAttribute("persistObject", Boolean.FALSE);
			return false;
		}
		
		ClientSrvInDTO 	srvInDTO 	= new ClientSrvInDTO();
		srvInDTO.setClientDTO(dto);		
		ClientSrvOutDTO srvOutDTO = this.clientService.getClient(srvInDTO);

		Boolean nou = true;
		if(srvOutDTO != null && srvOutDTO.getClientDTO() != null){
			nou = false;
			ClientDTO dto2 = srvOutDTO.getClientDTO();
			dto.setDataAlta(dto2.getDataAlta());
			srvInDTO.setClientDTO(dto);
		}
				
		if(nou){
			dto.setDataAlta(new Date(System.currentTimeMillis()));
			srvInDTO.setClientDTO(dto);
			String id = this.clientService.createClient(srvInDTO);
			//dto.setNif(id);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.alta", Missatge.TYPE_SUCCES));
			log.debug("id newClient--> " + id);
		}else{
			Boolean updated = this.clientService.updateClient(srvInDTO);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica", Missatge.TYPE_SUCCES));
			log.debug("updated Client--> " + updated);
		}
		
		request.setAttribute("dtoObject", dto);
		return true;
	}
	
	/**
	 * Carrega una entitat Client en request a partir del nif pasat tamb per request.
	 * @param request
	 * @throws Exception
	 */
	private void editar(PortletRequest request) throws Exception {
		log.debug("editar(request)");

		String nif 						= request.getParameter("nif");		
		ClientDTO dto					= new ClientDTO();
		dto.setNif(nif);
		
		ClientSrvInDTO 	srvInDTO 	= new ClientSrvInDTO();
		srvInDTO.setClientDTO(dto);
		
		ClientSrvOutDTO srvOutDTO = this.clientService.getClient(srvInDTO);
		dto = srvOutDTO.getClientDTO();
		request.setAttribute("dtoObject", dto);
	}
	
	/**
	 * Cerca un llistat d'entitats Client que compleixin els criteris i ordenaci passats per formulari request.
	 * El llistat resultant es passat per request.
	 * @param request
	 * @throws Exception
	 */
	private void buscar(PortletRequest request) throws Exception {
		log.debug("buscar(request)");

		String nif = request.getParameter("nif");
		String nom = request.getParameter("nom");
		String ordre = request.getParameter("ordre");

		//Provinientes de otra pagina
		String nif_gestoria = request.getParameter("nif_gestoria");
		GestoriaDTO gestoriaDTO = new GestoriaDTO();   
		gestoriaDTO.setNif(nif_gestoria);
		String nif_administrador = request.getParameter("nif_administrador");
		AdministradorDTO administradorDTO = new AdministradorDTO();
		administradorDTO.setNif(nif_administrador);
		
		ClientSrvInDTO 	srvInDTO 	= new ClientSrvInDTO();
		ClientDTO 		dto			= new ClientDTO();

		dto.setNif(nif);
		dto.setNom(nom);		
		dto.setGestoria(gestoriaDTO);
		dto.setAdministrador(administradorDTO);
		dto.setOrdre(ordre);
		request.setAttribute("dtoObject", dto);
		srvInDTO.setClientDTO(dto);

		ClientSrvOutDTO srvOutDTO = this.clientService.getClientsByCriteria(srvInDTO);
		Collection<ClientDTO> listaDTO = srvOutDTO.getClientDTOList();
		if(listaDTO == null || listaDTO.isEmpty()){
			request.setAttribute("missatges", Missatge.addMissatge("llistat.info.cerca", Missatge.TYPE_INFO));
		}
		if(listaDTO != null){
			for (ClientDTO clientDTO : listaDTO) {
				clientDTO.setColorSemaforFactura(this.clientService.getSemaforEstatFactures(clientDTO.getNif()));
				clientDTO.setColorSemaforIntervencio(this.clientService.getSemaforEstatIntervencions(clientDTO.getNif()));
			}
		}
		
		request.setAttribute("dtoList", listaDTO);		
		
		log.debug("listaDTO --> " + listaDTO);
	}	
	
	/**
	 * Valida els camps del formulari de l'entitat Client abans de guardar-los.
	 * Si troba errors son afegits per request.
	 * @param dto ClientDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaCamps(ClientDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valNull(dto.getNif(), missatges) 											&&
				ValidaCamps.valNull(dto.getNom(), missatges) 											&&
				ValidaCamps.valNull(dto.getCarrer(), missatges)											&&
				ValidaCamps.valNull(dto.getCp(), missatges)												&&
				ValidaCamps.valNull(dto.getTelefon(), missatges)										&&
				ValidaCamps.valNull(dto.getMail(), missatges)											&&
				ValidaCamps.valNull(dto.getTipusClient().getId(), missatges) 							&&
				ValidaCamps.valNIFCIF(dto.getNif(), missatges) 											&&
				ValidaCamps.valMinSize("CP", dto.getCp(), ClientVO.MIN_SIZE_CP, missatges) 	&&
				ValidaCamps.valMail(dto.getMail(), missatges)){
			return true;
		}else{
			if(	!ValidaCamps.valNull(dto.getNif(), new ArrayList<Missatge>()) ||
				!ValidaCamps.valNIFCIF(dto.getNif(), new ArrayList<Missatge>())){
				dto.setValidationNif(dto.getNif());
				dto.setNif(null);
			}			
			request.setAttribute("missatges", missatges);
			return false;
		}
	}


	/**
	 * @return the clientService
	 */
	public ClientService getClientService() {
		return clientService;
	}


	/**
	 * @param clientService the clientService to set
	 */
	public void setClientService(ClientService clientService) {
		this.clientService = clientService;
	}


	/**
	 * @return the gestoriaService
	 */
	public GestoriaService getGestoriaService() {
		return gestoriaService;
	}


	/**
	 * @param gestoriaService the gestoriaService to set
	 */
	public void setGestoriaService(GestoriaService gestoriaService) {
		this.gestoriaService = gestoriaService;
	}
	
}