package com.ferri.portlet.contracte.action;

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.TreeMap;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.web.context.ServletContextAware;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.Controller;
import org.springframework.web.portlet.mvc.SimpleFormController;

import com.ferri.persistence.bean.impl.ContracteVO;
import com.ferri.persistence.bean.impl.EstatVO;
import com.ferri.persistence.bean.impl.TipusClientVO;
import com.ferri.portlet.utils.Format;
import com.ferri.portlet.utils.MailUtils;
import com.ferri.portlet.utils.MessageResourceBundle;
import com.ferri.portlet.utils.Missatge;
import com.ferri.portlet.utils.PdfUtils;
import com.ferri.portlet.utils.ValidaCamps;
import com.ferri.service.dto.ClientDTO;
import com.ferri.service.dto.ConstantsDTO;
import com.ferri.service.dto.ContracteDTO;
import com.ferri.service.dto.EstatDTO;
import com.ferri.service.dto.FacturaDTO;
import com.ferri.service.dto.IntervencioDTO;
import com.ferri.service.dto.TipusPeriodeDTO;
import com.ferri.service.dto.input.ClientSrvInDTO;
import com.ferri.service.dto.input.ConstantsSrvInDTO;
import com.ferri.service.dto.input.ContracteSrvInDTO;
import com.ferri.service.dto.input.FacturaSrvInDTO;
import com.ferri.service.dto.input.IntervencioSrvInDTO;
import com.ferri.service.dto.output.ClientSrvOutDTO;
import com.ferri.service.dto.output.ContracteSrvOutDTO;
import com.ferri.service.dto.output.EstatSrvOutDTO;
import com.ferri.service.dto.output.TipusPeriodeSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.ClientService;
import com.ferri.service.interfaces.ContracteService;
import com.ferri.service.interfaces.FacturaService;

/**
 * Controlador de les pantalles de llistat.jsp i manteniment.jsp de la entitat Contracte
 * @author dferri
 *
 */
public class ViewContracteAction extends SimpleFormController implements Controller, ServletContextAware {
	private static final Log log = LogFactory.getLog(ViewContracteAction.class);

	private ContracteService 	contracteService	= null;
	private FacturaService 		facturaService 		= null;
	private ClientService 		clientService		= null;
	private ServletContext 		servletContext = null;

	private static final String PAGE_MANT	= "contracte/manteniment";
	private static final String PAGE_LIST	= "contracte/llista";
	
	public void handleActionRequest(ActionRequest request, ActionResponse response) throws Exception {	
	}
	
	
	public ModelAndView handleRenderRequest(RenderRequest request, RenderResponse response)	throws Exception {
		ModelAndView modView = null;
		
		String reqCode = request.getParameter("reqCode");
				
		if ((reqCode != null) && (!reqCode.equals(""))) {
			
			if (reqCode.equals("eliminar")){
				if(elimina(request)){
					modView = new ModelAndView(PAGE_LIST);
					subElements(request);
					defecte(request);
				}else{
					modView = new ModelAndView(PAGE_MANT);
					subElements(request);
					getClientsAutoComplete(request);
					getMails(request);
				}			
			}else if (reqCode.equals("afegir")) {
				modView = new ModelAndView(PAGE_MANT);	
				subElements(request);			
				getClientsAutoComplete(request);
			}else if(reqCode.equals("editar")) {
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				editar(request);
				getClientsAutoComplete(request);
				getMails(request);
				
			}else if (reqCode.equals("guardar")) {
				modView = new ModelAndView(PAGE_MANT);			
				subElements(request);
				request.setAttribute("result", guardar(request));
				getClientsAutoComplete(request);
				getMails(request);
				
			}else if (reqCode.equals("buscar")){
				modView = new ModelAndView(PAGE_LIST);
				subElements(request);
				buscar(request);
			}else if (reqCode.equals("enviar")){
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				enviar(request);
				getClientsAutoComplete(request);
			}else if (reqCode.equals("signar")){
				modView = new ModelAndView(PAGE_MANT);
				subElements(request);
				signar(request);
				getClientsAutoComplete(request);
				getMails(request);
			}
			
			System.out.println("----->" + reqCode);
		
		}else{			
			modView = new ModelAndView(PAGE_LIST);
			defecte(request);
			subElements(request);
		}
		
		return modView;
	}
	
	/**
	 * Carrega el llistat amb tots els elements Contracte i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void defecte(PortletRequest request) throws CommonServiceException{
		//Mostrar todos los elementos por defecto
		ContracteSrvOutDTO srvOutDTO = this.contracteService.getAllContracte();
		Collection<ContracteDTO> lista = srvOutDTO.getContracteDTOList();
		
		if (lista != null) {
			request.setAttribute("dtoList", lista);
		}		
	}
	
	/**
	 * Carrega els llistats d'elements secundaris: Estat, i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void subElements(PortletRequest request) throws CommonServiceException{				
		EstatSrvOutDTO estatSrvOutDTO = this.facturaService.getEstatsContracte();
		
		if(estatSrvOutDTO != null){
			Collection<EstatDTO> estatDTOs = estatSrvOutDTO.getEstatDTOList();
			request.setAttribute("llistaEstatDto", estatDTOs);
		}
		
	
		TipusPeriodeSrvOutDTO tipusPeriodeSrvOutDTO = this.contracteService.getAllTipusPeriode();
		if(tipusPeriodeSrvOutDTO != null){
			Collection<TipusPeriodeDTO> tipusPeriodeDTOs = tipusPeriodeSrvOutDTO.getTipusPeriodeDTOList();
			request.setAttribute("llistaPeriodeDto", tipusPeriodeDTOs);
		}
		
	}

	/**
	 * Carrega un String en el atribut request 'arrayClientAutocomplete' amb la relaci de NIF separada per comes de tots els clients amb la forma '"NIF1","NIF2","NIF3",...'.
	 * Es fa servir per generar l'autocomplete del camp client amb JQuery.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void getClientsAutoComplete(PortletRequest request) throws CommonServiceException{
		ClientSrvOutDTO clientSrvOutDTO = this.clientService.getAllClient();
		Collection<ClientDTO> clientDTOs = clientSrvOutDTO.getClientDTOList();
		
		StringBuffer arrayJquery = new StringBuffer();

		for (Iterator<ClientDTO> iterator = clientDTOs.iterator(); iterator.hasNext();) {
			ClientDTO clientDTO = iterator.next();
			arrayJquery.append("\"").append(clientDTO.getNif()).append("\"");
			if(iterator.hasNext())
				arrayJquery.append(",");				
		}
		request.setAttribute("arrayClientAutocomplete", arrayJquery.toString());
	}
	
	/**
	 * Metode que cerca els emails vinculats a un client, sempre el seu, i opcionalment el de la gestoria i l'administrador: sempre que es traci d'una comunitat.
	 * @param request amb la informaci, s'afegeis el llistat 'llistaMails' per referencia
	 * @throws CommonServiceException
	 */
	private void getMails(PortletRequest request) throws CommonServiceException{
		ArrayList<String[]> emails = new ArrayList<String[]>();
		
		ContracteDTO dto = (ContracteDTO) request.getAttribute("dtoObject");
		ClientDTO client = dto.getClient();
		
		if(client.getNif()!=null){
			client = this.clientService.getClient(new ClientSrvInDTO(client)).getClientDTO();
			
			String binomi1[] = {MessageResourceBundle.getString("popup.enviament.mail.desti1", request.getLocale()), client.getMail()}; 
			emails.add(binomi1);	
			
			if(client.getTipusClient().getId().intValue() == TipusClientVO.COMUNITAT){
				if(client.getGestoria()!=null && client.getGestoria().getMail()!=null){
					String binomi2[] = {MessageResourceBundle.getString("popup.enviament.mail.desti2", request.getLocale()), client.getGestoria().getMail()}; 
					emails.add(binomi2);
				}
				if(client.getAdministrador()!=null && client.getAdministrador().getMail()!=null){
					String binomi3[] = {MessageResourceBundle.getString("popup.enviament.mail.desti3", request.getLocale()), client.getAdministrador().getMail()}; 
					emails.add(binomi3);
				}
			}
		}
		request.setAttribute("llistaMails", emails);
	}
	
	/**
	 * Elimina una entitat Contracte amb la pk pasada en la request.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private boolean elimina(PortletRequest request) throws Exception {
		log.debug("delete(request)");

		//Long id = Long.valueOf(Long.parseLong(request.getParameter("id")));
		String id = String.valueOf((request.getParameter("id")));

		ContracteSrvInDTO 	srvInDTO 	= new ContracteSrvInDTO();
		ContracteDTO 		dto			= new ContracteDTO();

		dto.setNumContracte(id);
		srvInDTO.setContracteDTO(dto);
		Boolean deleted = false;
		try{
			deleted = this.contracteService.deleteContracte(srvInDTO);
		}catch(DataIntegrityViolationException e){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.elimina", Missatge.TYPE_ERROR));
			ContracteSrvOutDTO srvOutDTO = this.contracteService.getContracte(srvInDTO);
			dto = srvOutDTO.getContracteDTO();
			request.setAttribute("dtoObject", dto);
			return false;
		}
		if(deleted){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.elimina", Missatge.TYPE_SUCCES));
		}
		log.debug("deleted-->\t" + deleted);
		return deleted;
	}

	/**
	 * Guarda una entitat Contracte, previa validaci, amb tots els camps del forumlari pasats per request: pot crear-lo de nou si no existeix o actualitzar-lo si esxisteix.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private Boolean guardar(PortletRequest request) throws Exception {
		log.debug("guardar(request)");

		String num_contracte 	= request.getParameter("num_contracte");
		String periode			= request.getParameter("periode");
		String nif_client		= request.getParameter("nif_client");
		String base_imponible	= request.getParameter("base_imponible");
		
		ContracteDTO 		dto			= new ContracteDTO();

		dto.setNumContracte(num_contracte);
		dto.setBaseImponibleS(base_imponible);
		
		if(periode != null && !periode.trim().equals("")){
			TipusPeriodeDTO tipusPeriode = new TipusPeriodeDTO();
			tipusPeriode.setId(Long.parseLong(periode));
			dto.setTipusPeriode(tipusPeriode);
		}
		
		ClientDTO client = new ClientDTO();
		client.setNif(nif_client);
		dto.setClient(client);
		
		dto.setMesIni(Integer.parseInt(Format.MES_FORMAT.format(new Date(System.currentTimeMillis()))));
		
		if(!validaCamps(dto, request)){
			request.setAttribute("dtoObject", dto);
			return false;
		}
		
		//Validacion nif existente
		ClientSrvOutDTO clientSrvOutDTO = this.clientService.getClient(new ClientSrvInDTO(client));
		if(clientSrvOutDTO == null || clientSrvOutDTO.getClientDTO() == null){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.client.inexistent", Missatge.TYPE_ERROR));
			request.setAttribute("dtoObject", dto);
			return false;
		}
		
		dto.setBaseImponible(Double.valueOf(base_imponible.replaceAll(",", ".")));
		
		EstatDTO estatDTO = new EstatDTO();
		estatDTO.setId(EstatVO.CONTRACTE_PENDENT);
		dto.setEstat(estatDTO);
		
		Boolean nou = true;
		ContracteSrvInDTO srvInDTO 	= new ContracteSrvInDTO();
		if(num_contracte != null && !num_contracte.trim().equals("")){			
			srvInDTO.setContracteDTO(dto);		
			ContracteSrvOutDTO srvOutDTO = this.contracteService.getContracte(srvInDTO);
			if(srvOutDTO != null && srvOutDTO.getContracteDTO() != null){
				nou = false;
				ContracteDTO dto2 = srvOutDTO.getContracteDTO();
				dto.setData(dto2.getData());
				srvInDTO.setContracteDTO(dto);
			}
		}
		
		if(nou){
			dto.setNumContracte(this.contracteService.nextNumeroContracte());
			dto.setData(new Date(System.currentTimeMillis()));
			srvInDTO.setContracteDTO(dto);
			String id = this.contracteService.createContracte(srvInDTO);
			dto.setNumContracte(id);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.alta", Missatge.TYPE_SUCCES));
			
			log.debug("id newContracte--> " + id);
		}else{
			Boolean updated = this.contracteService.updateContracte(srvInDTO);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.contracte.estat1", Missatge.TYPE_SUCCES));
			log.debug("updated Contracte--> " + updated);
		}
		
		request.setAttribute("dtoObject", dto);
		return true;
	}

	/**
	 * Genera un pdf de l'entitat Contracte, i l'envia com attachment en un mail preformat d'aquesta entitat als destinataris passats per formulari en la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void enviar(PortletRequest request) throws CommonServiceException{
		String num_contracte = request.getParameter("id");
		String mails[] 	   = {request.getParameter("mail_0"), request.getParameter("mail_1"), request.getParameter("mail_2")};
		//Cargar
		ContracteDTO dto = new ContracteDTO();
		dto.setNumContracte(num_contracte);
		ContracteSrvInDTO srvInDTO = new ContracteSrvInDTO();
		srvInDTO.setContracteDTO(dto);		
		ContracteSrvOutDTO srvOutDTO = this.contracteService.getContracte(srvInDTO);
		dto = srvOutDTO.getContracteDTO();
		
		boolean enviat = false;
		
		//Generar impreso con dto		
		String xslPath 	= servletContext.getRealPath("xsl");			
		String xml		= dto.getXmlFop(xslPath);
	
		//Crear fichero temporal de trabajo
		String args[] = {dto.getNumContracte()};
		String fileName = MessageResourceBundle.getString("mail.attachment.name.contracte", request.getLocale(), args);
		
		try {
			String tmpPath 	= servletContext.getRealPath("tmp");
			File attachment = new File(tmpPath + File.separator + fileName);
			
			PdfUtils.generatePdf(xml, xslPath + File.separator + PdfUtils.XSL_CONTRACTE, attachment);
			
			String user = MessageResourceBundle.getString("mail.account.user", request.getLocale());
			String pass = MessageResourceBundle.getString("mail.account.hashcode", request.getLocale());
			
			String from = MessageResourceBundle.getString("mail.from.default", request.getLocale());
			
			String to[]	= mails;
			
			String subject = MessageResourceBundle.getString("mail.subject.contracte", request.getLocale(), args);
			
			String args2[] = {dto.getClient().getNom()};
			String body = 	MessageResourceBundle.getString("mail.body.inici", request.getLocale()) +
							MessageResourceBundle.getString("mail.body.centre.contracte", request.getLocale(), args2) +
							MessageResourceBundle.getString("mail.body.final", request.getLocale());
			
			MailUtils mailUtils = new MailUtils(user, pass);
			enviat = mailUtils.envioMail(from, to, subject, body, attachment);
			
			attachment.delete();
			
		} catch (Exception e) {
			e.printStackTrace();
			enviat = false;
		}
		
		
		//Cambiar estado y guardar
		EstatDTO estatDTO = new EstatDTO();
		estatDTO.setId(EstatVO.CONTRACTE_ENVIAT);
		if(dto.getEstat().getId() != null && dto.getEstat().getId().intValue() != EstatVO.CONTRACTE_SIGNAT.intValue()){
			dto.setEstat(estatDTO);
			srvInDTO.setContracteDTO(dto);
			this.contracteService.updateContracte(srvInDTO);
			if(enviat){
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.contracte.estat2", Missatge.TYPE_SUCCES));
			}else{
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.info.modifica.contracte.estat2", Missatge.TYPE_INFO));
			}
		}else{
			if(enviat){
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.enviament", Missatge.TYPE_SUCCES));
			}else{
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.enviament", Missatge.TYPE_ERROR));
			}
		}
		request.setAttribute("dtoObject", dto);
		
	}
	
	/**
	 * Cambia l'estat de l'entitat Contracte a signat, genera una entitat Factura en estat pendent amb les dades del Contracte, 
	 * tamb genera les entitats Intervencio associades al contracte en estat pendent.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void signar(PortletRequest request) throws CommonServiceException{
		String num_contracte = request.getParameter("id");
		//Cargar
		ContracteDTO dto = new ContracteDTO();
		dto.setNumContracte(num_contracte);
		ContracteSrvInDTO srvInDTO = new ContracteSrvInDTO();
		srvInDTO.setContracteDTO(dto);		
		ContracteSrvOutDTO srvOutDTO = this.contracteService.getContracte(srvInDTO);
		dto = srvOutDTO.getContracteDTO();

		//Cambiar estado y guardar
		EstatDTO estatDTO = new EstatDTO();
		estatDTO.setId(EstatVO.CONTRACTE_SIGNAT);
		dto.setEstat(estatDTO);
		srvInDTO.setContracteDTO(dto);
		this.contracteService.updateContracte(srvInDTO);
		request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.contracte.estat3", Missatge.TYPE_SUCCES));
		request.setAttribute("dtoObject", dto);
		
		//Creacin de la factura
		EstatDTO estatFactura = new EstatDTO();
		estatFactura.setId(EstatVO.FACTURA_PENDENT);
		
		FacturaDTO facturaDTO = new FacturaDTO();
		facturaDTO.setNumFactura(this.facturaService.nextNumeroFactura());
		facturaDTO.setData(new Date(System.currentTimeMillis()));
		
		int tipus = dto.getTipusPeriode().getId().intValue();
		ConstantsDTO constantsDTO = new ConstantsDTO();
		switch(tipus){
			case 1: //TipusPeriodeVO.ANUAL
				constantsDTO.setId(ContracteVO.CONTRACTE_ANUAL);
				constantsDTO = (this.facturaService.getConstants(new ConstantsSrvInDTO(constantsDTO))).getConstantsDTO();
				break;
			case 2: //TipusPeriodeVO.SEMESTRAL
				constantsDTO.setId(ContracteVO.CONTRACTE_SEMESTRAL);
				constantsDTO = (this.facturaService.getConstants(new ConstantsSrvInDTO(constantsDTO))).getConstantsDTO();
				break;
			case 3: //TipusPeriodeVO.TRIMESTRAL
				constantsDTO.setId(ContracteVO.CONTRACTE_TRIMESTRAL);
				constantsDTO = (this.facturaService.getConstants(new ConstantsSrvInDTO(constantsDTO))).getConstantsDTO();
				break;
			case 4: //TipusPeriodeVO.MENSUAL
				constantsDTO.setId(ContracteVO.CONTRACTE_MENSUAL);
				constantsDTO = (this.facturaService.getConstants(new ConstantsSrvInDTO(constantsDTO))).getConstantsDTO();
				break;
		}
		if(request.getLocale().getLanguage().startsWith("es")){
			facturaDTO.setConcepte(constantsDTO.getDescEs());
		}else{//ca
			facturaDTO.setConcepte(constantsDTO.getDescCa());
		}

		facturaDTO.setBaseImponible(dto.getBaseImponible());		
		facturaDTO.setEstat(estatFactura);
		facturaDTO.setClient(dto.getClient());
		this.facturaService.createFactura(new FacturaSrvInDTO(facturaDTO));				

		dto.setFactura(facturaDTO);
		srvInDTO.setContracteDTO(dto);
		this.contracteService.updateContracte(srvInDTO);
		
		//Creacion de las intervenciones derivdas del contrato
		this.creaIntervencions(dto);
	}
	
	
	/**
	 * Retorna un binomi amb la informaci mes/any agafant com a base el mes actual y afegint les unitats de temps pasades.
	 * @param field Unitat de temps de Calendar
	 * @param amount valor a incrementar
	 * @return Integer[] binomi
	 */
	private Integer[] getMesAny(int field, int amount){
		Calendar today = Calendar.getInstance();
		today.add(field, amount);
		Integer mes = Integer.parseInt(Format.MES_FORMAT.format(today.getTime()));
		Integer any = Integer.parseInt(Format.ANY_FORMAT.format(today.getTime()));
		
		Integer[] retorn = {mes ,any};
		
		return retorn;
	}
	
	/**
	 * Crea les entitats Intervencio en estat pendent a partir d'una entitat Contracte
	 * @param dto ContracteDTO
	 * @throws CommonServiceException
	 */
	private void creaIntervencions(ContracteDTO dto) throws CommonServiceException{
		
		TreeMap<Integer, Integer[]> preIntervencions = new TreeMap<Integer, Integer[]>();  

		switch(dto.getTipusPeriode().getId().intValue()){

		case 4: //TipusPeriodeVO.MENSUAL:
			preIntervencions.put(2, getMesAny(Calendar.MONTH, 2));
			preIntervencions.put(3, getMesAny(Calendar.MONTH, 3));
			preIntervencions.put(5, getMesAny(Calendar.MONTH, 5));
			preIntervencions.put(6, getMesAny(Calendar.MONTH, 6));
			preIntervencions.put(8, getMesAny(Calendar.MONTH, 8));
			preIntervencions.put(9, getMesAny(Calendar.MONTH, 9));
			preIntervencions.put(11, getMesAny(Calendar.MONTH, 11));
			preIntervencions.put(12, getMesAny(Calendar.MONTH, 12));
		case 3: //TipusPeriodeVO.TRIMESTRAL:
			preIntervencions.put(4, getMesAny(Calendar.MONTH, 4));
			preIntervencions.put(10, getMesAny(Calendar.MONTH, 10));
		case 2: //TipusPeriodeVO.SEMESTRAL:
			preIntervencions.put(7, getMesAny(Calendar.MONTH, 7));
		case 1: //TipusPeriodeVO.ANUAL:
			preIntervencions.put(1, getMesAny(Calendar.MONTH, 1));
		}
		
		EstatDTO estatIntervencio = new EstatDTO();
		estatIntervencio.setId(EstatVO.INTERVENCIO_PENDENT);
		
		ContracteSrvInDTO contracteSrvInDTO = null;
		IntervencioDTO intervencioDTO = null;
		for(Iterator<Integer> keys = preIntervencions.keySet().iterator(); keys.hasNext();){
			Integer[] mesAny = preIntervencions.get(keys.next());
			
			intervencioDTO = new IntervencioDTO();
			intervencioDTO.setAny(mesAny[1]);
			intervencioDTO.setMes(mesAny[0]);
			intervencioDTO.setContracte(dto);
			intervencioDTO.setClient(dto.getClient());
			intervencioDTO.setEstat(estatIntervencio);
			this.contracteService.createIntervencio(new IntervencioSrvInDTO(intervencioDTO));
		}
	}
	
	/**
	 * Carrega una entitat Contracte en request a partir del nmero pasat tamb per request.
	 * @param request
	 * @throws Exception
	 */
	private void editar(PortletRequest request) throws Exception {
		log.debug("editar(request)");

		String numContracte 				= request.getParameter("num_contracte");		
		ContracteDTO dto					= new ContracteDTO();
		dto.setNumContracte(numContracte);
		
		ContracteSrvInDTO 	srvInDTO 	= new ContracteSrvInDTO();
		srvInDTO.setContracteDTO(dto);
		
		ContracteSrvOutDTO srvOutDTO = this.contracteService.getContracte(srvInDTO);
		dto = srvOutDTO.getContracteDTO();
		request.setAttribute("dtoObject", dto);
	}
	
	/**
	 * Cerca un llistat d'entitats Contracte que compleixin els criteris i ordenaci passats per formulari request.
	 * El llistat resultant es passat per request.
	 * @param request
	 * @throws Exception
	 */
	private void buscar(PortletRequest request) throws Exception {
		log.debug("buscar(request)");

		String num_contracte 	= request.getParameter("num_contracte");
		String nif_client		= request.getParameter("nif_client");
		String periode			= request.getParameter("periode");
		String estat			= request.getParameter("estat");
		String ordre 			= request.getParameter("ordre");
		//Para validar
		String dataDesde		= request.getParameter("dataDesde");
		String dataFins			= request.getParameter("dataFins");
		
		//Tiene que venir uno de estos 4 valores:
		//entitat.client, entitat.data 
		//entitat.data
		
		
		ContracteSrvInDTO 	srvInDTO 	= new ContracteSrvInDTO();
		ContracteDTO 		dto			= new ContracteDTO();

		dto.setNumContracte(num_contracte);
		
		if(estat != null && !estat.trim().equals("")){
			EstatDTO estatDTO = new EstatDTO();
			estatDTO.setId(Long.parseLong(estat));
			dto.setEstat(estatDTO);
		}
		
		if(periode != null && !periode.trim().equals("")){
			TipusPeriodeDTO tipusPeriode = new TipusPeriodeDTO();
			tipusPeriode.setId(Long.parseLong(periode));
			dto.setTipusPeriode(tipusPeriode);
		}
		
		ClientDTO client = new ClientDTO();
		client.setNif(nif_client);
		dto.setClient(client);
		
		dto.setDataIniS(dataDesde);
		dto.setDataFinS(dataFins);
		
		dto.setOrdre(ordre);

		if(!validaBusqueda(dto, request)){
			request.setAttribute("dtoObject", dto);
			request.setAttribute("dtoList", null);
			return;
		}
		
		if(dataDesde != null && !dataDesde.trim().equals(""))
			dto.setDataIni(Format.DATE_FORMAT.parse(dataDesde));
		
		if(dataFins != null && !dataFins.trim().equals(""))
			dto.setDataFin(Format.DATE_FORMAT.parse(dataFins));
		
		request.setAttribute("dtoObject", dto);
		srvInDTO.setContracteDTO(dto);

		ContracteSrvOutDTO srvOutDTO = this.contracteService.getContractesByCriteria(srvInDTO);
		Collection<ContracteDTO> listaDTO = srvOutDTO.getContracteDTOList();
		if(listaDTO == null || listaDTO.isEmpty()){
			request.setAttribute("missatges", Missatge.addMissatge("llistat.info.cerca", Missatge.TYPE_INFO));
		}
		request.setAttribute("dtoList", listaDTO);		
		
		log.debug("listaDTO --> " + listaDTO);
	}	
	
	/**
	 * Valida els camps del formulari de l'entitat Contracte abans de guardar-los.
	 * Si troba errors son afegits per request.
	 * @param dto ContracteDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaCamps(ContracteDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valNull(dto.getClient().getNif(), missatges) &&
				ValidaCamps.valNull(dto.getBaseImponibleS(), missatges)	 &&
				ValidaCamps.valNumber("Base Imponible", dto.getBaseImponibleS(), missatges)){
			return true;
		}else{
			request.setAttribute("missatges", missatges);
			return false;
		}
	}

	/**
	 * Valida els camps del formulari de cerca en format data per entitats Contracte.
	 * @param dto ContracteDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaBusqueda(ContracteDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valFecha(dto.getDataIniS(), missatges)	&&
				ValidaCamps.valFecha(dto.getDataFinS(), missatges)	){
			
			return true;
		}else{
			request.setAttribute("missatges", missatges);
			return false;
		}
	}

	
	/**
	 * @return the facturaService
	 */
	public FacturaService getFacturaService() {
		return facturaService;
	}


	/**
	 * @param facturaService the facturaService to set
	 */
	public void setFacturaService(FacturaService facturaService) {
		this.facturaService = facturaService;
	}


	/**
	 * @return the contracteService
	 */
	public ContracteService getContracteService() {
		return contracteService;
	}


	/**
	 * @param contracteService the contracteService to set
	 */
	public void setContracteService(ContracteService contracteService) {
		this.contracteService = contracteService;
	}


	/**
	 * @return the clientService
	 */
	public ClientService getClientService() {
		return clientService;
	}


	/**
	 * @param clientService the clientService to set
	 */
	public void setClientService(ClientService clientService) {
		this.clientService = clientService;
	}


	/**
	 * @param servletContext the servletContext to set
	 */
	public void setServletContext(ServletContext servletContext) {
		this.servletContext = servletContext;
	}
	
}