package com.ferri.portlet.factura.action;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.context.ServletContextAware;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.Controller;
import org.springframework.web.portlet.mvc.SimpleFormController;

import com.ferri.persistence.bean.impl.ConstantsVO;
import com.ferri.persistence.bean.impl.EstatVO;
import com.ferri.persistence.bean.impl.TipusClientVO;
import com.ferri.portlet.utils.Format;
import com.ferri.portlet.utils.MailUtils;
import com.ferri.portlet.utils.MessageResourceBundle;
import com.ferri.portlet.utils.Missatge;
import com.ferri.portlet.utils.PdfUtils;
import com.ferri.portlet.utils.ValidaCamps;
import com.ferri.service.dto.ClientDTO;
import com.ferri.service.dto.ConstantsDTO;
import com.ferri.service.dto.EstatDTO;
import com.ferri.service.dto.FacturaDTO;
import com.ferri.service.dto.input.ClientSrvInDTO;
import com.ferri.service.dto.input.ConstantsSrvInDTO;
import com.ferri.service.dto.input.FacturaSrvInDTO;
import com.ferri.service.dto.output.ClientSrvOutDTO;
import com.ferri.service.dto.output.EstatSrvOutDTO;
import com.ferri.service.dto.output.FacturaSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.ClientService;
import com.ferri.service.interfaces.FacturaService;

/**
 * Controlador de les pantalles de llistat.jsp i manteniment.jsp de la entitat Factura
 * @author dferri
 *
 */
public class ViewFacturaAction extends SimpleFormController implements Controller, ServletContextAware{
	
	private static final Log log = LogFactory.getLog(ViewFacturaAction.class);

	private FacturaService facturaService = null;
	private ClientService clientService = null;	
	private ServletContext servletContext = null;
	
	private static final String PAGE_MANT	= "factura/manteniment";
	private static final String PAGE_LIST	= "factura/llista";
	
	public void handleActionRequest(ActionRequest request, ActionResponse response) throws Exception {	
		//String reqCode = request.getParameter("reqCode");
	}
	
	
	public ModelAndView handleRenderRequest(RenderRequest request, RenderResponse response)	throws Exception {
		ModelAndView modView = null;
		
		String reqCode = request.getParameter("reqCode");
				
		if ((reqCode != null) && (!reqCode.equals(""))) {
			
			if (reqCode.equals("afegir")) {
				modView = new ModelAndView(PAGE_MANT);
				getClientsAutoComplete(request);
			
			}else if(reqCode.equals("editar")) {
				modView = new ModelAndView(PAGE_MANT);
				editar(request);
				getClientsAutoComplete(request);
				getMails(request);
				
			}else if (reqCode.equals("guardar")) {
				modView = new ModelAndView(PAGE_MANT);				
				request.setAttribute("result", guardar(request));
				getClientsAutoComplete(request);
				getMails(request);
				
			}else if (reqCode.equals("buscar")){
				modView = new ModelAndView(PAGE_LIST);
				buscar(request);
			}else if (reqCode.equals("enviar")){
				modView = new ModelAndView(PAGE_MANT);
				enviar(request);
				getClientsAutoComplete(request);
				getMails(request);
			}else if (reqCode.equals("cobrar")){
				modView = new ModelAndView(PAGE_MANT);
				cobrar(request);
				getClientsAutoComplete(request);
				getMails(request);
			}
			
			//Siempre se tiene que cargar:
			subElements(request);
			System.out.println("----->" + reqCode);
		
		}else{			
			modView = new ModelAndView(PAGE_LIST);
			defecte(request);
			subElements(request);
		}
		
		return modView;
	}
	
	/**
	 * Carrega el llistat amb tots els elements Factura i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void defecte(PortletRequest request) throws CommonServiceException{
		//Mostrar todos los elementos por defecto
		FacturaSrvOutDTO srvOutDTO = this.facturaService.getAllFactura();
		Collection<FacturaDTO> lista = srvOutDTO.getFacturaDTOList();
		
		if (lista != null) {
			request.setAttribute("dtoList", lista);
			request.getPortletSession().setAttribute("llistaCercaExcel", lista, PortletSession.APPLICATION_SCOPE);
		}		
	}
	
	/**
	 * Carrega un String en el atribut request 'arrayClientAutocomplete' amb la relaci de NIF separada per comes de tots els clients amb la forma '"NIF1","NIF2","NIF3",...'.
	 * Es fa servir per generar l'autocomplete del camp client amb JQuery.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void getClientsAutoComplete(PortletRequest request) throws CommonServiceException{
		ClientSrvOutDTO clientSrvOutDTO = this.clientService.getAllClient();
		Collection<ClientDTO> clientDTOs = clientSrvOutDTO.getClientDTOList();
		
		StringBuffer arrayJquery = new StringBuffer();

		for (Iterator<ClientDTO> iterator = clientDTOs.iterator(); iterator.hasNext();) {
			ClientDTO clientDTO = iterator.next();
			arrayJquery.append("\"").append(clientDTO.getNif()).append("\"");
			if(iterator.hasNext())
				arrayJquery.append(",");				
		}
		request.setAttribute("arrayClientAutocomplete", arrayJquery.toString());
	}
	
	/**
	 * Metode que cerca els emails vinculats a un client, sempre el seu, i opcionalment el de la gestoria i l'administrador: sempre que es traci d'una comunitat.
	 * @param request amb la informaci, s'afegeis el llistat 'llistaMails' per referencia
	 * @throws CommonServiceException
	 */
	private void getMails(PortletRequest request) throws CommonServiceException{
		ArrayList<String[]> emails = new ArrayList<String[]>();
		
		FacturaDTO dto = (FacturaDTO) request.getAttribute("dtoObject");
		ClientDTO client = dto.getClient();
		
		if(client.getNif()!=null){
			client = this.clientService.getClient(new ClientSrvInDTO(client)).getClientDTO();
			
			String binomi1[] = {MessageResourceBundle.getString("popup.enviament.mail.desti1", request.getLocale()), client.getMail()}; 
			emails.add(binomi1);	
			
			if(client.getTipusClient().getId().intValue() == TipusClientVO.COMUNITAT){
				if(client.getGestoria()!=null && client.getGestoria().getMail()!=null){
					String binomi2[] = {MessageResourceBundle.getString("popup.enviament.mail.desti2", request.getLocale()), client.getGestoria().getMail()}; 
					emails.add(binomi2);
				}
				if(client.getAdministrador()!=null && client.getAdministrador().getMail()!=null){
					String binomi3[] = {MessageResourceBundle.getString("popup.enviament.mail.desti3", request.getLocale()), client.getAdministrador().getMail()}; 
					emails.add(binomi3);
				}
			}
		}
		request.setAttribute("llistaMails", emails);
	}
	

	/**
	 * Guarda una entitat Factura, previa validaci, amb tots els camps del forumlari pasats per request: pot crear-lo de nou si no existeix o actualitzar-lo si esxisteix.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private Boolean guardar(PortletRequest request) throws Exception {
		log.debug("guardar(request)");

		String num_factura 		= request.getParameter("num_factura");
		String base_imponible	= request.getParameter("base_imponible");
		String concepte			= request.getParameter("concepte");
		String nif_client		= request.getParameter("nif_client");
		
		//String data				= request.getParameter("data");
		//String data_cobrament				= request.getParameter("data_cobrament");
		
		FacturaDTO 		dto			= new FacturaDTO();

		dto.setNumFactura(num_factura);
		dto.setBaseImponibleS(base_imponible);
		dto.setConcepte(concepte);
		
		ClientDTO client = new ClientDTO();
		client.setNif(nif_client);
		dto.setClient(client);
		
		if(!validaCamps(dto, request)){			
			request.setAttribute("dtoObject", dto);
			return false;
		}
		
		//Validacion nif existente
		ClientSrvOutDTO clientSrvOutDTO = this.clientService.getClient(new ClientSrvInDTO(client));
		if(clientSrvOutDTO == null || clientSrvOutDTO.getClientDTO() == null){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.client.inexistent", Missatge.TYPE_ERROR));
			request.setAttribute("dtoObject", dto);
			return false;
		}
		
		dto.setBaseImponible(Double.valueOf(base_imponible.replaceAll(",", ".")));
		
		EstatDTO estatDTO = new EstatDTO();		
		estatDTO.setId(EstatVO.FACTURA_PENDENT);		
		dto.setEstat(estatDTO);
		
		Boolean nou = true;
		FacturaSrvInDTO srvInDTO 	= new FacturaSrvInDTO();
		if(num_factura != null && !num_factura.trim().equals("")){			
			srvInDTO.setFacturaDTO(dto);		
			FacturaSrvOutDTO srvOutDTO = this.facturaService.getFactura(srvInDTO);
			if(srvOutDTO != null && srvOutDTO.getFacturaDTO() != null){
				nou = false;
				FacturaDTO dto2 = srvOutDTO.getFacturaDTO();
				dto.setData(dto2.getData());
				srvInDTO.setFacturaDTO(dto);
			}
		}
		
		if(nou){
			dto.setNumFactura(this.facturaService.nextNumeroFactura());
			dto.setData(new Date(System.currentTimeMillis()));
			srvInDTO.setFacturaDTO(dto);
			String id = this.facturaService.createFactura(srvInDTO);
			dto.setNumFactura(id);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.alta", Missatge.TYPE_SUCCES));
			
			log.debug("id newFactura--> " + id);
		}else{
			Boolean updated = this.facturaService.updateFactura(srvInDTO);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.factura.estat1", Missatge.TYPE_SUCCES));
			log.debug("updated Factura--> " + updated);
		}
		
		request.setAttribute("dtoObject", dto);
		return true;
	}
	
	/**
	 * Genera un pdf de l'entitat Factura, i l'envia com attachment en un mail preformat d'aquesta entitat als destinataris passats per formulari en la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void enviar(PortletRequest request) throws CommonServiceException{
		String num_factura = request.getParameter("id");
		String mails[] 	   = {request.getParameter("mail_0"), request.getParameter("mail_1"), request.getParameter("mail_2")};
		//Cargar factura
		FacturaDTO dto = new FacturaDTO();
		dto.setNumFactura(num_factura);
		FacturaSrvInDTO srvInDTO = new FacturaSrvInDTO();
		srvInDTO.setFacturaDTO(dto);		
		FacturaSrvOutDTO srvOutDTO = this.facturaService.getFactura(srvInDTO);
		dto = srvOutDTO.getFacturaDTO();

		//Recuperar tipos impositivos
		ConstantsSrvInDTO constantsSrvInDTO = new ConstantsSrvInDTO();
		ConstantsDTO constantsDTO = new ConstantsDTO();
		
		constantsDTO.setId(ConstantsVO.KEY_IVA);
		constantsSrvInDTO.setConstantsDTO(constantsDTO);		
		Double percentIVA = this.facturaService.getConstants(constantsSrvInDTO).getConstantsDTO().getValor();
		
		constantsDTO.setId(ConstantsVO.KEY_IRPF);
		constantsSrvInDTO.setConstantsDTO(constantsDTO);
		Double percentIRPF = this.facturaService.getConstants(constantsSrvInDTO).getConstantsDTO().getValor();
		
		boolean enviat = false;
		
		//Generar impreso con dto		
		String xslPath 	= servletContext.getRealPath("xsl");			
		String xml		= dto.getXmlFop(xslPath, percentIVA, percentIRPF);
	
		//Crear fichero temporal de trabajo
		String args[] = {dto.getNumFactura()};
		String fileName = MessageResourceBundle.getString("mail.attachment.name.factura", request.getLocale(), args);
		
		try {
			String tmpPath 	= servletContext.getRealPath("tmp");
			File attachment = new File(tmpPath + File.separator + fileName);
			
			PdfUtils.generatePdf(xml, xslPath + File.separator + PdfUtils.XSL_FACTURA, attachment);
			
			String user = MessageResourceBundle.getString("mail.account.user", request.getLocale());
			String pass = MessageResourceBundle.getString("mail.account.hashcode", request.getLocale());
			
			String from = MessageResourceBundle.getString("mail.from.default", request.getLocale());
			
			String to[]	= mails;			
			
			String subject = MessageResourceBundle.getString("mail.subject.factura", request.getLocale(), args);
			
			String args2[] = {dto.getClient().getNom()};
			String body = 	MessageResourceBundle.getString("mail.body.inici", request.getLocale()) +
							MessageResourceBundle.getString("mail.body.centre.factura", request.getLocale(), args2) +
							MessageResourceBundle.getString("mail.body.final", request.getLocale());
			
			MailUtils mailUtils = new MailUtils(user, pass);
			enviat = mailUtils.envioMail(from, to, subject, body, attachment);
			
			attachment.delete();
			
		} catch (Exception e) {
			e.printStackTrace();
			enviat = false;
		}
		
		//Cambiar estado y guardar
		EstatDTO estatDTO = new EstatDTO();
		estatDTO.setId(EstatVO.FACTURA_ENVIADA);
		if(dto.getEstat().getId() != null && dto.getEstat().getId().intValue() != EstatVO.FACTURA_COBRADA.intValue()){
			dto.setEstat(estatDTO);
			srvInDTO.setFacturaDTO(dto);
			this.facturaService.updateFactura(srvInDTO);
			if(enviat){
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.factura.estat2", Missatge.TYPE_SUCCES));
			}else{
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.info.modifica.factura.estat2", Missatge.TYPE_INFO));
			}
		}else{
			if(enviat){
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.enviament", Missatge.TYPE_SUCCES));
			}else{
				request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.enviament", Missatge.TYPE_ERROR));
			}
		}
		request.setAttribute("dtoObject", dto);
		
	}
	
	/**
	 * Cambia l'estat de l'entitat Factura a Cobrada.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void cobrar(PortletRequest request) throws CommonServiceException{
		String num_factura = request.getParameter("id");
		//Cargar factura
		FacturaDTO dto = new FacturaDTO();
		dto.setNumFactura(num_factura);
		FacturaSrvInDTO srvInDTO = new FacturaSrvInDTO();
		srvInDTO.setFacturaDTO(dto);		
		FacturaSrvOutDTO srvOutDTO = this.facturaService.getFactura(srvInDTO);
		dto = srvOutDTO.getFacturaDTO();

		//Cambiar estado y guardar
		EstatDTO estatDTO = new EstatDTO();
		estatDTO.setId(EstatVO.FACTURA_COBRADA);
		dto.setEstat(estatDTO);
		dto.setDataCobrament(new Date(System.currentTimeMillis()));
		
		srvInDTO.setFacturaDTO(dto);
		this.facturaService.updateFactura(srvInDTO);
		request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.factura.estat3", Missatge.TYPE_SUCCES));
		request.setAttribute("dtoObject", dto);
	}
	
	/**
	 * Carrega una entitat Factura en request a partir del nmero pasat tamb per request.
	 * @param request
	 * @throws Exception
	 */
	private void editar(PortletRequest request) throws Exception {
		log.debug("editar(request)");

		String numFactura 				= request.getParameter("num_factura");		
		FacturaDTO dto					= new FacturaDTO();
		dto.setNumFactura(numFactura);
		
		FacturaSrvInDTO 	srvInDTO 	= new FacturaSrvInDTO();
		srvInDTO.setFacturaDTO(dto);
		
		FacturaSrvOutDTO srvOutDTO = this.facturaService.getFactura(srvInDTO);
		dto = srvOutDTO.getFacturaDTO();
		request.setAttribute("dtoObject", dto);
	}
	
	/**
	 * Cerca un llistat d'entitats Factura que compleixin els criteris i ordenaci passats per formulari request.
	 * El llistat resultant es passat per request.
	 * @param request
	 * @throws Exception
	 */
	private void buscar(PortletRequest request) throws Exception {
		log.debug("buscar(request)");

		String num_factura 		= request.getParameter("num_factura");
		String nif_client		= request.getParameter("nif_client");
		String concepte			= request.getParameter("concepte");
		String estat			= request.getParameter("estat");
		String ordre 			= request.getParameter("ordre");
		//Para validar
		String dataDesde		= request.getParameter("dataDesde");
		String dataFins			= request.getParameter("dataFins");
		String importDesde		= request.getParameter("importDesde");
		String importFins		= request.getParameter("importFins");
		
		//Tiene que venir uno de estos 4 valores:
		//entitat.baseImponible, entitat.data
		//entitat.data, entitat.baseImponible		
		//entitat.client.nom, entitat.data
		//entitat.client.nom, entitat.baseImponible
		
		
		
		FacturaSrvInDTO 	srvInDTO 	= new FacturaSrvInDTO();
		FacturaDTO 		dto			= new FacturaDTO();

		dto.setNumFactura(num_factura);
		dto.setConcepte(concepte);
		
		if(estat != null && !estat.equals("")){
			EstatDTO estatDTO = new EstatDTO();
			estatDTO.setId(Long.parseLong(estat));
			dto.setEstat(estatDTO);
		}
		
		ClientDTO client = new ClientDTO();
		client.setNif(nif_client);
		dto.setClient(client);
		
		dto.setDataIniS(dataDesde);
		dto.setDataFinS(dataFins);
		dto.setBaseIniS(importDesde);
		dto.setBaseFinS(importFins);
		
		dto.setOrdre(ordre);

		if(!validaBusqueda(dto, request)){
			request.setAttribute("dtoObject", dto);
			request.setAttribute("dtoList", null);
			return;
		}
		if(dataDesde != null && !dataDesde.trim().equals(""))
			dto.setDataIni(Format.DATE_FORMAT.parse(dataDesde));
		if(dataFins != null && !dataFins.trim().equals(""))
			dto.setDataFin(Format.DATE_FORMAT.parse(dataFins));
		
		if(importDesde != null && !importDesde.trim().equals(""))
			dto.setBaseIni(Double.valueOf(importDesde.replace(",", ".")));
		if(importFins != null && !importFins.trim().equals(""))
			dto.setBaseFin(Double.valueOf(importFins.replace(",", ".")));
		
		request.setAttribute("dtoObject", dto);
		srvInDTO.setFacturaDTO(dto);

		FacturaSrvOutDTO srvOutDTO = this.facturaService.getFacturasByCriteria(srvInDTO);
		Collection<FacturaDTO> listaDTO = srvOutDTO.getFacturaDTOList();
		if(listaDTO == null || listaDTO.isEmpty()){
			request.setAttribute("missatges", Missatge.addMissatge("llistat.info.cerca", Missatge.TYPE_INFO));
		}
		request.setAttribute("dtoList", listaDTO);
		//Adjuntem la llista per la generaci del Excel
		request.getPortletSession().setAttribute("llistaCercaExcel", listaDTO, PortletSession.APPLICATION_SCOPE);		
		
		log.debug("listaDTO --> " + listaDTO);
	}	
	
	/**
	 * Carrega els llistats d'elements secundaris: Estat, i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void subElements(PortletRequest request) throws CommonServiceException{				
		EstatSrvOutDTO estatSrvOutDTO = this.facturaService.getEstatsFactura();
		
		if(estatSrvOutDTO != null){
			Collection<EstatDTO> estatDTOs = estatSrvOutDTO.getEstatDTOList();
			request.setAttribute("llistaEstatDto", estatDTOs);
		}
	}
	
	/**
	 * Valida els camps del formulari de l'entitat Factura abans de guardar-los.
	 * Si troba errors son afegits per request.
	 * @param dto FacturaDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaCamps(FacturaDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valNull(dto.getBaseImponibleS(), missatges)			&&
				ValidaCamps.valNull(dto.getConcepte(), missatges)				&&
				ValidaCamps.valNull(dto.getClient().getNif(), missatges) 		&&
				ValidaCamps.valNumber("Base Imponible", dto.getBaseImponibleS(), missatges)){
			return true;
		}else{
			request.setAttribute("missatges", missatges);
			return false;
		}
	}

	/**
	 * Valida els camps del formulari de cerca en format data o import per entitats Factura.
	 * @param dto FacturaDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaBusqueda(FacturaDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valFecha(dto.getDataIniS(), missatges)						&&
				ValidaCamps.valFecha(dto.getDataFinS(), missatges)						&&
				ValidaCamps.valNumber("Base Imponible", dto.getBaseIniS(), missatges)	&&
				ValidaCamps.valNumber("Base Imponible", dto.getBaseFinS(), missatges)	){
			
			return true;
		}else{
			request.setAttribute("missatges", missatges);
			return false;
		}
	}

	/**
	 * @return the facturaService
	 */
	public FacturaService getFacturaService() {
		return facturaService;
	}


	/**
	 * @param facturaService the facturaService to set
	 */
	public void setFacturaService(FacturaService facturaService) {
		this.facturaService = facturaService;
	}


	/**
	 * @return the clientService
	 */
	public ClientService getClientService() {
		return clientService;
	}


	/**
	 * @param clientService the clientService to set
	 */
	public void setClientService(ClientService clientService) {
		this.clientService = clientService;
	}

	/*
	 * (non-Javadoc)
	 * @see org.springframework.web.context.ServletContextAware#setServletContext(javax.servlet.ServletContext)
	 */
	public void setServletContext(ServletContext servletContext) {
		this.servletContext = servletContext;		
	}

}