package com.ferri.portlet.gestoria.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.Controller;

import com.ferri.persistence.bean.impl.GestoriaVO;
import com.ferri.portlet.utils.Missatge;
import com.ferri.portlet.utils.ValidaCamps;
import com.ferri.service.dto.GestoriaDTO;
import com.ferri.service.dto.input.GestoriaSrvInDTO;
import com.ferri.service.dto.output.GestoriaSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.GestoriaService;

/**
 * Controlador de les pantalles de llistat.jsp i manteniment.jsp de la entitat Gestoria
 * @author dferri
 *
 */
public class ViewGestoriaAction implements Controller {
	private static final Log log = LogFactory.getLog(ViewGestoriaAction.class);

	private GestoriaService gestoriaService = null;

	private static final String PAGE_MANT	= "gestoria/manteniment";
	private static final String PAGE_LIST	= "gestoria/llista";
	
	public void handleActionRequest(ActionRequest request, ActionResponse response) throws Exception {	
		//String reqCode = request.getParameter("reqCode");
	}
	
	
	public ModelAndView handleRenderRequest(RenderRequest request, RenderResponse response)	throws Exception {
		ModelAndView modView = null;
		
		String reqCode = request.getParameter("reqCode");
				
		if ((reqCode != null) && (!reqCode.equals(""))) {
			
			if (reqCode.equals("eliminar")){
				if(elimina(request)){
					modView = new ModelAndView(PAGE_LIST);
					defecte(request);
				}else{
					modView = new ModelAndView(PAGE_MANT);
				}			
			}else if (reqCode.equals("afegir")) {
				modView = new ModelAndView(PAGE_MANT);				
			
			}else if(reqCode.equals("editar")) {
				modView = new ModelAndView(PAGE_MANT);
				editar(request);
				
			}else if (reqCode.equals("guardar")) {
				modView = new ModelAndView(PAGE_MANT);				
				request.setAttribute("result", guardar(request));
				
			}else if (reqCode.equals("buscar")){
				modView = new ModelAndView(PAGE_LIST);
				buscar(request);
			}
			
			System.out.println("----->" + reqCode);
		
		}else{			
			modView = new ModelAndView(PAGE_LIST);
			defecte(request);
		}
		
		return modView;
	}
	
	/**
	 * Carrega el llistat amb tots els elements Gestoria i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void defecte(PortletRequest request) throws CommonServiceException{
		//Mostrar todos los elementos por defecto
		GestoriaSrvOutDTO srvOutDTO = this.gestoriaService.getAllGestoria();
		Collection<GestoriaDTO> lista = srvOutDTO.getGestoriaDTOList();
		if(lista != null){
			for (GestoriaDTO gestoriaDTO : lista) {
				gestoriaDTO.setNumClients(this.gestoriaService.countClients(gestoriaDTO.getNif()));
				gestoriaDTO.setNumAdministradors(this.gestoriaService.countAdrministradors(gestoriaDTO.getNif()));
			}
		
			request.setAttribute("dtoList", lista);
		}		
	}
	
	/**
	 * Elimina una entitat Gestoria amb la pk pasada en la request.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private boolean elimina(PortletRequest request) throws Exception {
		log.debug("delete(request)");

		//Long id = Long.valueOf(Long.parseLong(request.getParameter("id")));
		String id = String.valueOf((request.getParameter("id")));

		GestoriaSrvInDTO 	srvInDTO 	= new GestoriaSrvInDTO();
		GestoriaDTO 		dto			= new GestoriaDTO();

		dto.setNif(id);
		srvInDTO.setGestoriaDTO(dto);
		Boolean deleted = false;
		try{
			deleted = this.gestoriaService.deleteGestoria(srvInDTO);
		}catch(DataIntegrityViolationException e){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.error.elimina", Missatge.TYPE_ERROR));
			GestoriaSrvOutDTO srvOutDTO = this.gestoriaService.getGestoria(srvInDTO);
			dto = srvOutDTO.getGestoriaDTO();
			request.setAttribute("dtoObject", dto);
			return false;
		}
		if(deleted){
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.elimina", Missatge.TYPE_SUCCES));
		}
		log.debug("deleted-->\t" + deleted);
		return deleted;
	}

	/**
	 * Guarda una entitat Gestoria, previa validaci, amb tots els camps del forumlari pasats per request: pot crear-lo de nou si no existeix o actualitzar-lo si esxisteix.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private Boolean guardar(PortletRequest request) throws Exception {
		log.debug("guardar(request)");

		String nif 				= request.getParameter("nif");
		String nom 				= request.getParameter("nom");
		String carrer 			= request.getParameter("carrer");
		String numero 			= request.getParameter("numero");
		String resta 			= request.getParameter("resta");
		String municipi 		= request.getParameter("municipi");
		String cp 				= request.getParameter("cp");
		String nom_comptable 	= request.getParameter("nom_comptable");
		String nif_comptable	= request.getParameter("nif_comptable");
		String telefon 			= request.getParameter("telefon");
		String mail 			= request.getParameter("mail");
		String comissio			= request.getParameter("comissio");
		
		
		GestoriaDTO 		dto			= new GestoriaDTO();
		
		dto.setNif(nif);
		dto.setNom(nom);
		dto.setCarrer(carrer);
		dto.setNumero(numero);
		dto.setResta(resta);
		dto.setMunicipi(municipi);
		dto.setCp(cp);
		dto.setNomComptable(nom_comptable);
		dto.setNifComptable(nif_comptable);
		dto.setTelefon(telefon);
		dto.setMail(mail);
		dto.setComissio(comissio);
		dto.setDataModif(new Date(System.currentTimeMillis()));

		if(!validaCamps(dto, request)){			
			request.setAttribute("dtoObject", dto);
			request.setAttribute("persistObject", Boolean.FALSE);
			return false;
		}
		
		GestoriaSrvInDTO 	srvInDTO 	= new GestoriaSrvInDTO();
		srvInDTO.setGestoriaDTO(dto);		
		GestoriaSrvOutDTO srvOutDTO = this.gestoriaService.getGestoria(srvInDTO);

		Boolean nou = true;
		if(srvOutDTO != null && srvOutDTO.getGestoriaDTO() != null){
			nou = false;
			GestoriaDTO dto2 = srvOutDTO.getGestoriaDTO();
			dto.setDataAlta(dto2.getDataAlta());
			srvInDTO.setGestoriaDTO(dto);
		}
				
		if(nou){
			dto.setDataAlta(new Date(System.currentTimeMillis()));
			srvInDTO.setGestoriaDTO(dto);
			String id = this.gestoriaService.createGestoria(srvInDTO);
			//dto.setNif(id);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.alta", Missatge.TYPE_SUCCES));
			log.debug("id newGestoria--> " + id);
		}else{
			Boolean updated = this.gestoriaService.updateGestoria(srvInDTO);
			request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica", Missatge.TYPE_SUCCES));
			log.debug("updated Gestoria--> " + updated);
		}
		
		request.setAttribute("dtoObject", dto);
		return true;
	}
	
	/**
	 * Carrega una entitat Gestoria en request a partir del nif pasat tamb per request.
	 * @param request
	 * @throws Exception
	 */
	private void editar(PortletRequest request) throws Exception {
		log.debug("afegir(request)");

		String nif 				= request.getParameter("nif");		
		GestoriaDTO 		dto			= new GestoriaDTO();
		dto.setNif(nif);
		GestoriaSrvInDTO 	srvInDTO 	= new GestoriaSrvInDTO();
		srvInDTO.setGestoriaDTO(dto);		
		GestoriaSrvOutDTO srvOutDTO = this.gestoriaService.getGestoria(srvInDTO);
		dto = srvOutDTO.getGestoriaDTO();
		request.setAttribute("dtoObject", dto);
	}
	
	/**
	 * Cerca un llistat d'entitats Gestoria que compleixin els criteris i ordenaci passats per formulari request.
	 * El llistat resultant es passat per request.
	 * @param request
	 * @throws Exception
	 */
	private void buscar(PortletRequest request) throws Exception {
		log.debug("buscar(request)");

		String nif = request.getParameter("nif");
		String nom = request.getParameter("nom");
		String ordre = request.getParameter("ordre");

		GestoriaSrvInDTO 	srvInDTO 	= new GestoriaSrvInDTO();
		GestoriaDTO 		dto			= new GestoriaDTO();

		dto.setNif(nif);
		dto.setNom(nom);
		dto.setOrdre(ordre);
		request.setAttribute("dtoObject", dto);
		srvInDTO.setGestoriaDTO(dto);

		GestoriaSrvOutDTO srvOutDTO = this.gestoriaService.getGestoriasByCriteria(srvInDTO);
		Collection<GestoriaDTO> listaDTO = srvOutDTO.getGestoriaDTOList();
		if(listaDTO == null || listaDTO.isEmpty()){
			request.setAttribute("missatges", Missatge.addMissatge("llistat.info.cerca", Missatge.TYPE_INFO));
		}
		if(listaDTO != null){
			for (GestoriaDTO gestoriaDTO : listaDTO) {
				gestoriaDTO.setNumClients(this.gestoriaService.countClients(gestoriaDTO.getNif()));
				gestoriaDTO.setNumAdministradors(this.gestoriaService.countAdrministradors(gestoriaDTO.getNif()));
			}
		}
		request.setAttribute("dtoList", listaDTO);		
		
		log.debug("listaDTO --> " + listaDTO);
	}	
	
	
	/**
	 * Valida els camps del formulari de l'entitat Gestoria abans de guardar-los.
	 * Si troba errors son afegits per request.
	 * @param dto GestoriaDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaCamps(GestoriaDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valNull(dto.getNif(), missatges) 											&&
				ValidaCamps.valNull(dto.getNom(), missatges) 											&&
				ValidaCamps.valNull(dto.getCarrer(), missatges)											&&
				ValidaCamps.valNull(dto.getNumero(), missatges)											&&
				ValidaCamps.valNull(dto.getResta(), missatges)											&&
				ValidaCamps.valNull(dto.getMunicipi(), missatges)										&&
				ValidaCamps.valNull(dto.getCp(), missatges)												&&
				ValidaCamps.valNull(dto.getNomComptable(), missatges)									&&
				ValidaCamps.valNull(dto.getNifComptable(), missatges)									&&
				ValidaCamps.valNull(dto.getTelefon(), missatges)										&&
				ValidaCamps.valNull(dto.getMail(), missatges)											&&
				ValidaCamps.valNull(dto.getComissio(), missatges) 										&&
				ValidaCamps.valNIFCIF(dto.getNif(), missatges) 											&&
				ValidaCamps.valMinSize("CP", dto.getCp(), GestoriaVO.MIN_SIZE_CP, missatges) 	&&
				ValidaCamps.valNIFCIF(dto.getNifComptable(), missatges) 								&&
				ValidaCamps.valMail(dto.getMail(), missatges)){			
			return true;
		}else{			
			if(	!ValidaCamps.valNull(dto.getNif(), new ArrayList<Missatge>()) ||
				!ValidaCamps.valNIFCIF(dto.getNif(), new ArrayList<Missatge>())){
				dto.setValidationNif(dto.getNif());
				dto.setNif(null);
			}			
			request.setAttribute("missatges", missatges);
			return false;
		}
	}
	
	/**
	 * @return the gestoriaService
	 */
	public GestoriaService getGestoriaService() {
		return gestoriaService;
	}

	/**
	 * @param gestoriaService the gestoriaService to set
	 */
	public void setGestoriaService(GestoriaService gestoriaService) {
		this.gestoriaService = gestoriaService;
	}
	
}