package com.ferri.portlet.intervencio.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.Controller;

import com.ferri.persistence.bean.impl.EstatVO;
import com.ferri.portlet.utils.Format;
import com.ferri.portlet.utils.Missatge;
import com.ferri.portlet.utils.ValidaCamps;
import com.ferri.service.dto.ClientDTO;
import com.ferri.service.dto.ContracteDTO;
import com.ferri.service.dto.EstatDTO;
import com.ferri.service.dto.IntervencioDTO;
import com.ferri.service.dto.input.IntervencioSrvInDTO;
import com.ferri.service.dto.output.EstatSrvOutDTO;
import com.ferri.service.dto.output.IntervencioSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.ContracteService;
import com.ferri.service.interfaces.FacturaService;

/**
 * Controlador de les pantalles de llistat.jsp de la entitat Intervencio
 * @author dferri
 *
 */
public class ViewIntervencioAction implements Controller{
	private static final Log log = LogFactory.getLog(ViewIntervencioAction.class);

	private ContracteService contracteService = null;
	private FacturaService facturaService = null;

	private static final String PAGE_LIST	= "intervencio/llista";
	
	public void handleActionRequest(ActionRequest request, ActionResponse response) throws Exception {	
	}
	
	
	public ModelAndView handleRenderRequest(RenderRequest request, RenderResponse response)	throws Exception {
		ModelAndView modView = null;
		
		String reqCode = request.getParameter("reqCode");
				
		if ((reqCode != null) && (!reqCode.equals(""))) {
			
			if (reqCode.equals("guardar")) {
				modView = new ModelAndView(PAGE_LIST);			
				subElements(request);
				request.setAttribute("result", guardar(request));
				defecte(request);
				
			}else if (reqCode.equals("buscar")){
				modView = new ModelAndView(PAGE_LIST);
				subElements(request);
				buscar(request);
			}
			
			System.out.println("----->" + reqCode);
		
		}else{			
			modView = new ModelAndView(PAGE_LIST);
			subElements(request);
			defecte(request);
		}
		
		return modView;
	}
	
	/**
	 * Carrega el llistat amb tots els elements Intervencio i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void defecte(PortletRequest request) throws CommonServiceException{
		//Mostrar todos los elementos por defecto
		IntervencioSrvOutDTO srvOutDTO = this.contracteService.getAllIntervencio();
		Collection<IntervencioDTO> lista = srvOutDTO.getIntervencioDTOList();
		
		if (lista != null) {
			request.setAttribute("dtoList", lista);
		}		
	}

	/**
	 * Carrega els llistats d'elements secundaris: Estat, i l'afegeix a la request.
	 * @param request
	 * @throws CommonServiceException
	 */
	private void subElements(PortletRequest request) throws CommonServiceException{				
		EstatSrvOutDTO estatSrvOutDTO = this.facturaService.getEstatsIntervencio();
		
		if(estatSrvOutDTO != null){
			Collection<EstatDTO> estatDTOs = estatSrvOutDTO.getEstatDTOList();
			request.setAttribute("llistaEstatDto", estatDTOs);
		}
	}
	
	/**
	 * Actualitza una entitat Intervencio a estat Realitzada.
	 * @param request
	 * @return boolean resultat
	 * @throws Exception
	 */
	private Boolean guardar(PortletRequest request) throws Exception {
		log.debug("guardar(request)");

		String id 		= request.getParameter("id");
		
		IntervencioDTO	dto	= new IntervencioDTO();
		dto.setId(Long.parseLong(id));
		
		IntervencioSrvInDTO srvInDTO 	= new IntervencioSrvInDTO();
		srvInDTO.setIntervencioDTO(dto);		
		IntervencioSrvOutDTO srvOutDTO = this.contracteService.getIntervencio(srvInDTO);
		dto = srvOutDTO.getIntervencioDTO();
		
		EstatDTO estatDTO = new EstatDTO();
		estatDTO.setId(EstatVO.INTERVENCIO_REALITZADA);
		dto.setEstat(estatDTO);
		
		dto.setDataIntervencio(new Date(System.currentTimeMillis()));
		
		srvInDTO.setIntervencioDTO(dto);		
		Boolean updated = this.contracteService.updateIntervencio(srvInDTO);
			
		request.setAttribute("missatges", Missatge.addMissatge("manteniments.succes.modifica.intervencio.estat2", Missatge.TYPE_SUCCES));
			
		log.debug("updated Intervencio--> " + updated);
		
		request.setAttribute("dtoObject", null);
		return true;
	}
	
	/**
	 * Cerca un llistat d'entitats Intervencio que compleixin els criteris i ordenaci passats per formulari request.
	 * El llistat resultant es passat per request.
	 * @param request
	 * @throws Exception
	 */
	private void buscar(PortletRequest request) throws Exception {
		log.debug("buscar(request)");

		String num_contracte 	= request.getParameter("num_contracte");
		String nif_client		= request.getParameter("nif_client");
		String estat			= request.getParameter("estat");
		String ordre 			= request.getParameter("ordre");
		//Para validar
		String dataDesde		= request.getParameter("dataDesde");
		String dataFins			= request.getParameter("dataFins");
		
		//Tiene que venir uno de estos valores:
		//entitat.client, entitat.data 
		//entitat.data
		
		IntervencioSrvInDTO srvInDTO 	= new IntervencioSrvInDTO();
		IntervencioDTO 		dto			= new IntervencioDTO();

		ContracteDTO contracteDTO = new ContracteDTO();
		contracteDTO.setNumContracte(num_contracte);
		dto.setContracte(contracteDTO);
		
		if(estat != null && !estat.equals("")){
			EstatDTO estatDTO = new EstatDTO();
			estatDTO.setId(Long.parseLong(estat));
			dto.setEstat(estatDTO);
		}
		
		ClientDTO client = new ClientDTO();
		client.setNif(nif_client);
		dto.setClient(client);
		
		dto.setDataIniS(dataDesde);
		dto.setDataFinS(dataFins);
		
		dto.setOrdre(ordre);

		if(!validaBusqueda(dto, request)){
			request.setAttribute("dtoObject", dto);
			request.setAttribute("dtoList", null);
			return;
		}
		
		if(dataDesde != null && !dataDesde.trim().equals(""))
			dto.setMesAnyIni(Format.DATE_FORMAT.parse(Format.DEFAULT_DIA + dataDesde.trim()));
		
		if(dataFins != null && !dataFins.trim().equals(""))
			dto.setMesAnyFin(Format.DATE_FORMAT.parse(Format.DEFAULT_DIA + dataFins.trim()));
		
		request.setAttribute("dtoObject", dto);
		srvInDTO.setIntervencioDTO(dto);

		IntervencioSrvOutDTO srvOutDTO = this.contracteService.getIntervenciosByCriteria(srvInDTO);
		Collection<IntervencioDTO> listaDTO = srvOutDTO.getIntervencioDTOList();
		if(listaDTO == null || listaDTO.isEmpty()){
			request.setAttribute("missatges", Missatge.addMissatge("llistat.info.cerca", Missatge.TYPE_INFO));
		}
		request.setAttribute("dtoList", listaDTO);		
		
		log.debug("listaDTO --> " + listaDTO);
	}	
	
	/**
	 * Valida els camps del formulari de cerca en format mes/any o import per entitats Intervencio.
	 * @param dto IntervencioDTO
	 * @param request
	 * @return boolean resultat
	 */
	private boolean validaBusqueda(IntervencioDTO dto, PortletRequest request){
		
		ArrayList<Missatge> missatges = new ArrayList<Missatge>();
		
		if(
				ValidaCamps.valMesAny(dto.getDataIniS(), missatges)						&&
				ValidaCamps.valMesAny(dto.getDataFinS(), missatges)						){
			
			return true;
		}else{
			request.setAttribute("missatges", missatges);
			return false;
		}
	}
	
	/**
	 * @return the contracteService
	 */
	public ContracteService getContracteService() {
		return contracteService;
	}


	/**
	 * @param contracteService the contracteService to set
	 */
	public void setContracteService(ContracteService contracteService) {
		this.contracteService = contracteService;
	}


	/**
	 * @return the facturaService
	 */
	public FacturaService getFacturaService() {
		return facturaService;
	}


	/**
	 * @param facturaService the facturaService to set
	 */
	public void setFacturaService(FacturaService facturaService) {
		this.facturaService = facturaService;
	}

}