package com.ferri.portlet.utils;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.Date;
import java.util.Locale;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import com.ferri.persistence.bean.impl.TipusClientVO;
import com.ferri.service.dto.FacturaDTO;

/**
 * Classe on delega la generaci de documents Excel per factures amb les llibreries POI.
 * @author dferri
 *
 */
public class ExcelUtils {

	private static final Integer COLUMNA_DESTACADA_08 = 8;
	private static final Integer COLUMNA_DESTACADA_10 = 10;
	private static final Integer COLUMNA_DESTACADA_11 = 11;
	
	/**
	 * Genera un Excel de la selecci de factures psada en memoria per obrir en request
	 * @param factures llista amb la selecci
	 * @param percentIVA
	 * @param percentIRPF
	 * @param locale
	 * @param osExcel
	 * @throws IOException
	 */
	public static Integer generaExcelFactura(Collection<FacturaDTO> factures, Double percentIVA, Double percentIRPF, Locale locale, OutputStream osExcel) throws IOException {
		//Generacin del liblo xls
		HSSFWorkbook workbook = new HSSFWorkbook();
		
		//Creacin de la hoja
		HSSFSheet sheet = workbook.createSheet("fulla");
		sheet.setDefaultColumnWidth((short)40);
		//sheet.setColumnWidth((short)0, (short)5000); 
		
		
		HSSFRow row = null;
		row = sheet.createRow((short)0);
		HSSFCell cell = row.createCell((short)0);
		HSSFRichTextString richTextString = null;
		
		String args[] = {Format.date2String(new Date(System.currentTimeMillis()))};
		String titolFulla = MessageResourceBundle.getString("excel.facura.titol", locale, args);
		
		richTextString = new HSSFRichTextString(titolFulla);
		cell.setCellType(HSSFCell.CELL_TYPE_STRING);
		cell.setCellValue(richTextString);		
		ompleCelda(cell, workbook, richTextString, titolFulla, false, true);
		
		//linea en blanco para separar ttulo de contenido
		row = sheet.createRow((short)1);
		cell = row.createCell((short)0);
		cell.setCellType(HSSFCell.CELL_TYPE_STRING);
		ompleCelda(cell, workbook, richTextString, "", false, false);
		
		
		int rowIdx = 2;
		int cellIdx = 0;
		
		String[] headers = {"excel.facura.header.data", 
							"excel.facura.header.numero",
							"excel.facura.header.nif",
							"excel.facura.header.cp",
							"excel.facura.header.client",
							"excel.facura.header.concepte",
							"excel.facura.header.bi",
							"excel.facura.header.iva1",
							"excel.facura.header.iva2",
							"excel.facura.header.ret1",
							"excel.facura.header.ret2",
							"excel.facura.header.total"};
							
		//Pintem les capaleres
		row = sheet.createRow((short)rowIdx);
		for (String key : headers) {							
			String valor = MessageResourceBundle.getString(key, locale);								
			cell = row.createCell((short)cellIdx);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			if(cellIdx == COLUMNA_DESTACADA_08 || cellIdx == COLUMNA_DESTACADA_10 || cellIdx == COLUMNA_DESTACADA_11){
				ompleCelda(cell, workbook, richTextString, valor, true, true);
			}else{
				ompleCelda(cell, workbook, richTextString, valor, false, true);
			}
			cellIdx ++;
		}
		rowIdx ++;
		
		cellIdx = 0;
		for (FacturaDTO facturaDTO : factures) {			
			row = sheet.createRow((short)rowIdx);
		
			String data 	= Format.date2String(facturaDTO.getData());
			String concepte = facturaDTO.getConcepte() != null && facturaDTO.getConcepte().length() > 15 ? facturaDTO.getConcepte().substring(0, 13) + "..." : facturaDTO.getConcepte();
			Double importe  = facturaDTO.getBaseImponible();			
			Double iva2 	= facturaDTO.getIVA(percentIVA);
			Double irpf1	= facturaDTO.getClient().getTipusClient().getId().intValue() == TipusClientVO.EMPRESA.intValue() ? percentIRPF : 0.0;			
			Double irpf2 	= facturaDTO.getClient().getTipusClient().getId().intValue() == TipusClientVO.EMPRESA.intValue() ? facturaDTO.getIRPF(percentIRPF) : 0.0;
			Double total	= importe + iva2 - irpf2;
			
			//Data
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			ompleCelda(cell, workbook, richTextString, data, false, false);
			//Numfactura
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			ompleCelda(cell, workbook, richTextString, facturaDTO.getNumFactura(), false, false);
			//nif
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			ompleCelda(cell, workbook, richTextString, facturaDTO.getClient().getNif(), false, false);
			//cp
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			ompleCelda(cell, workbook, richTextString, facturaDTO.getClient().getCp(), false, false);
			//nom client
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			ompleCelda(cell, workbook, richTextString, facturaDTO.getClient().getNom(), false, false);
			//concepte
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			ompleCelda(cell, workbook, richTextString, concepte, false, false);
			//imoport
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
			ompleCelda(cell, workbook, richTextString, Format.IMPORTS_FORMAT.format(importe), true, false);
			//%iva
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
			ompleCelda(cell, workbook, richTextString, Format.IMPORTS_FORMAT.format(percentIVA), false, false);			
			//imp iva
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
			ompleCelda(cell, workbook, richTextString, Format.IMPORTS_FORMAT.format(iva2), true, false);
			//%irpf
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
			ompleCelda(cell, workbook, richTextString, Format.IMPORTS_FORMAT.format(irpf1), false, false);
			//imp irpf
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
			ompleCelda(cell, workbook, richTextString, Format.IMPORTS_FORMAT.format(irpf2), true, false);
			//imp total
			cell = row.createCell((short)cellIdx++);
			cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
			ompleCelda(cell, workbook, richTextString, Format.IMPORTS_FORMAT.format(total), true, true);
			cellIdx = 0;
			rowIdx ++;
		}					
		
		//Escrivim OS		
		workbook.write(osExcel);		
		osExcel.flush();
		osExcel.close();
		return workbook.getBytes().length;		
	}
	

	/**
	 * Introdueix un valor amb a la cella passada amb les caracterstiques indicades.
	 * @param pCell HSSFCell a tractar
	 * @param pWorkbook HSSFWorkbook
	 * @param pRichTextString HSSFRichTextString
	 * @param pTexto String amb el contingut
	 * @param destaca boolean remarcar en vermell
	 * @param esCap boolean remarcar en negreta
	 */
	private static void ompleCelda(HSSFCell pCell, HSSFWorkbook pWorkbook, HSSFRichTextString pRichTextString, String pTexto, boolean destaca, boolean esCap) {
		
		HSSFRichTextString richTextString = new HSSFRichTextString(pTexto);
		
		// establecemos el tipo de fuente
		HSSFFont font = pWorkbook.createFont();
		if(esCap)
			font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
		
		if(destaca)
			font.setColor(HSSFFont.COLOR_RED);
		
		// creamos el objeto que se encargar de aplicar el estilo a la celda
		HSSFCellStyle cellStyle = pWorkbook.createCellStyle();
		cellStyle.setBorderBottom((short)0);
		cellStyle.setFont(font);
		
		pCell.setCellStyle(cellStyle);
		pCell.setCellValue(richTextString);
	}
	
}
