package com.ferri.portlet.utils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.BodyPart;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

/**
 * Calsse on deleguen les utilitats de mailing.
 * @author dferri
 *
 */
public class MailUtils {
	
	private String userAccount 	= null;
	private String hashcode		= null;

	/**
	 * Creacio MailUtils 
	 * @param userAccount
	 * @param hashcode
	 */
	public MailUtils(String userAccount, String hashcode){
		this.userAccount = userAccount;
		this.hashcode = hashcode;
	}
	
	/**
	 * Envia un mail amb una compte gmail preconfigurat
	 * @param from String
	 * @param to String[]
	 * @param subject String
	 * @param body String amb format html
	 * @param attachment File
	 * @return boolean resultat
	 */
	public boolean envioMail(String from, String[] to, String subject, String body, File attachment){
		
		Properties props = new Properties();
		 
		 props.put("mail.transport.protocol", 	"smtp");
		 props.put("mail.smtp.starttls.enable", "true");
		 props.put("mail.smtp.host", 			"smtp.gmail.com");
		 props.setProperty("mail.smtp.auth", 	"true");      
		
		
		Session mailSession = Session.getDefaultInstance(props, null);
		mailSession.setDebug(true);

		Transport transport;
		
		try {
			
			transport = mailSession.getTransport( );

			MimeMessage message = new MimeMessage(mailSession);

			message.setSubject(subject);
			message.setFrom(new InternetAddress(from));
			
			for (String desti : to) {
				if(desti != null && !desti.equals(""))
					message.addRecipient(Message.RecipientType.TO, new InternetAddress(desti));
			}

//	      Create the message part 
			BodyPart messageBodyPart = new MimeBodyPart();

//	         Fill the message
			messageBodyPart.setText(subject);
			messageBodyPart.setContent(body, "text/html");
			Multipart multipart = new MimeMultipart();
			multipart.addBodyPart(messageBodyPart);

//	         Part two is attachment
			messageBodyPart = new MimeBodyPart();
			DataSource source = new FileDataSource(attachment);
			messageBodyPart.setDataHandler(new DataHandler(source));
			messageBodyPart.setFileName(attachment.getName());
			multipart.addBodyPart(messageBodyPart);

//	         Put parts in message
			message.setContent(multipart);

			transport.connect("smtp.gmail.com",getUserAccount(),getHashcode());			

			transport.sendMessage(message,message.getRecipients(Message.RecipientType.TO));
			transport.close();
	        
		} catch (MessagingException e) {
			return false;
		}
	        return true;
	}


    /*
     * Inner class to act as a JAF datasource to send HTML e-mail content
     */
    static class HTMLDataSource implements DataSource {
        private String html;

        public HTMLDataSource(String htmlString) {
            html = htmlString;
        }

        // Return html string in an InputStream.
        // A new stream must be returned each time.
        public InputStream getInputStream() throws IOException {
            if (html == null) throw new IOException("Null HTML");
            return new ByteArrayInputStream(html.getBytes());
        }

        public OutputStream getOutputStream() throws IOException {
            throw new IOException("This DataHandler cannot write HTML");
        }

        public String getContentType() {
            return "text/html";
        }

        public String getName() {
            return "JAF text/html dataSource to send e-mail only";
        }
    }


	/**
	 * @return the userAccount
	 */
	public String getUserAccount() {
		return userAccount;
	}


	/**
	 * @param userAccount the userAccount to set
	 */
	public void setUserAccount(String userAccount) {
		this.userAccount = userAccount;
	}


	/**
	 * @return the hashcode
	 */
	public String getHashcode() {
		return hashcode;
	}


	/**
	 * @param hashcode the hashcode to set
	 */
	public void setHashcode(String hashcode) {
		this.hashcode = hashcode;
	}

}
