package com.ferri.portlet.utils;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.OutputStream;
import java.io.StringReader;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;

/**
 * Classe on delega la generaci de documents PDF amb les llibreries FOP.
 * @author dferri
 *
 */
public class PdfUtils {
	
	public static final String PDF_TIPUS_FACTURA 	= "F";
	public static final String PDF_TIPUS_PRESSUPOST	= "P";
	public static final String PDF_TIPUS_CONTRACTE	= "C";
	
	public static final String XSL_FACTURA 		= "factura.xsl";
	public static final String XSL_PRESSUPOST	= "pressupost.xsl";
	public static final String XSL_CONTRACTE	= "contracte.xsl";

	/**
	 * Genera un Pdf en memoria per obrir en request
	 * @param xml
	 * @param xslPath
	 * @param osPdf
	 */
	public static void generatePdf(String xml, String xslPath, OutputStream osPdf){
		
		try{			
//			Setup input and output files
            
            File xsltfile = new File(xslPath);
            // configure fopFactory as desired
            FopFactory fopFactory = FopFactory.newInstance();

            FOUserAgent foUserAgent = fopFactory.newFOUserAgent();
            // configure foUserAgent as desired

            // Setup output
            osPdf = new BufferedOutputStream(osPdf);
            
                // Construct fop with desired output format
                Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF, foUserAgent, osPdf);

                // Setup XSLT
                TransformerFactory factory = TransformerFactory.newInstance();
                Transformer transformer = factory.newTransformer(new StreamSource(xsltfile));

                // Set the value of a <param> in the stylesheet
                transformer.setParameter("versionParam", "2.0");

                // Setup input for XSLT transformation
                Source src = new StreamSource(new StringReader(xml));

                // Resulting SAX events (the generated FO) must be piped through to FOP
                Result res = new SAXResult(fop.getDefaultHandler());

                // Start XSLT transformation and FOP processing
                transformer.transform(src, res);
            
                osPdf.flush();            

            System.out.println("Success!");
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
	}
	
	/**
	 * Genera Pdf en fitxer per enviar com attachment
	 * @param xml
	 * @param xslPath
	 * @param pdffile
	 */
	public static void generatePdf(String xml, String xslPath, File pdffile){
		
		try{			
//			Setup input and output files
            
            File xsltfile = new File(xslPath);
            // configure fopFactory as desired
            FopFactory fopFactory = FopFactory.newInstance();

            FOUserAgent foUserAgent = fopFactory.newFOUserAgent();
            // configure foUserAgent as desired

            // Setup output
            OutputStream outt = new java.io.FileOutputStream(pdffile);
            outt = new BufferedOutputStream(outt);

            
                // Construct fop with desired output format
                Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF, foUserAgent, outt);

                // Setup XSLT
                TransformerFactory factory = TransformerFactory.newInstance();
                Transformer transformer = factory.newTransformer(new StreamSource(xsltfile));

                // Set the value of a <param> in the stylesheet
                transformer.setParameter("versionParam", "2.0");

                // Setup input for XSLT transformation
                Source src = new StreamSource(new StringReader(xml));

                // Resulting SAX events (the generated FO) must be piped through to FOP
                Result res = new SAXResult(fop.getDefaultHandler());

                // Start XSLT transformation and FOP processing
                transformer.transform(src, res);
            
                outt.close();            

            System.out.println("Success!");
        } catch (Exception e) {
            e.printStackTrace(System.err);
            
        }
	}
	
}