package com.ferri.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.beanutils.BeanUtils;

import com.ferri.persistence.bean.impl.AdministradorVO;
import com.ferri.persistence.bean.impl.ClientVO;
import com.ferri.persistence.bean.impl.TipusClientVO;
import com.ferri.persistence.dao.interfaces.ClientDAO;
import com.ferri.persistence.dao.interfaces.TipusClientDAO;
import com.ferri.service.commons.GenericBaseServiceImpl;
import com.ferri.service.dto.AdministradorDTO;
import com.ferri.service.dto.ClientDTO;
import com.ferri.service.dto.TipusClientDTO;
import com.ferri.service.dto.input.ClientSrvInDTO;
import com.ferri.service.dto.input.TipusClientSrvInDTO;
import com.ferri.service.dto.output.AdministradorSrvOutDTO;
import com.ferri.service.dto.output.ClientSrvOutDTO;
import com.ferri.service.dto.output.TipusClientSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.ClientService;

public class ClientServiceImpl extends GenericBaseServiceImpl implements ClientService {
	private ClientDAO 		clientDAO 		= null;
	private TipusClientDAO 	tipusClientDAO	= null;

	/**
	 * 
	 */
	public Boolean updateClient(ClientSrvInDTO clientSrvInDTO) throws CommonServiceException {
		try {
			ClientVO tmpClientVO = (ClientVO)getBeansConverter().convert(clientSrvInDTO.getClientDTO());
			ClientVO clientVO = (ClientVO) getClientDAO().load(tmpClientVO.getNif());

			BeanUtils.copyProperties(clientVO, tmpClientVO);

			getClientDAO().update(clientVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String createClient(ClientSrvInDTO clientSrvInDTO) throws CommonServiceException
	{
		try {
			ClientDTO clientDTO = clientSrvInDTO.getClientDTO();

			ClientVO clientVO = (ClientVO)getBeansConverter().convert(clientDTO);

			String id = (String) getClientDAO().create(clientVO);
			clientDTO.setNif(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deleteClient(ClientSrvInDTO clientSrvInDTO) throws CommonServiceException
	{
		try {
			getClientDAO().removeByPk(clientSrvInDTO.getClientDTO().getNif());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public ClientSrvOutDTO getClient(ClientSrvInDTO clientSrvInDTO) throws CommonServiceException
	{
		try {
			ClientVO clientVO = (ClientVO)getClientDAO().load(clientSrvInDTO.getClientDTO().getNif());
			return new ClientSrvOutDTO((ClientDTO)getBeansConverter().convert(clientVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public ClientSrvOutDTO getClientsByCriteria(ClientSrvInDTO clientSrvInDTO) throws CommonServiceException {

		try{			
			Collection<ClientVO> clientVoCol = null;

			StringBuffer query = new StringBuffer("from ClientVO entitat where 1=1");
			if(clientSrvInDTO !=null && clientSrvInDTO.getClientDTO() != null){
				ClientDTO dto = clientSrvInDTO.getClientDTO();
				if(dto.getNif() != null && !dto.getNif().trim().equals("")){
					query.append(" and upper(entitat.nif) like upper('").append(dto.getNif()).append("%')");
				}
				if(dto.getNom() != null && !dto.getNom().trim().equals("")){
					query.append(" and upper(entitat.nom) like upper('%").append(dto.getNom()).append("%')");
				}
				if(dto.getGestoria() != null && dto.getGestoria().getNif() != null && !dto.getGestoria().getNif().equals("")){
					query.append(" and upper(entitat.gestoria.nif) like upper('").append(dto.getGestoria().getNif()).append("%')");
				}
				if(dto.getAdministrador() != null && dto.getAdministrador().getNif() != null && !dto.getAdministrador().getNif().equals("")){
					query.append(" and upper(entitat.administrador.nif) like upper('").append(dto.getAdministrador().getNif()).append("%')");
				}
				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					query.append(" order by entitat.").append(dto.getOrdre());
				}
			}
			clientVoCol = getClientDAO().find(query.toString(), null);

			return new ClientSrvOutDTO((Collection<ClientDTO>) getBeansConverter().convertCollection(clientVoCol)); 
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public ClientSrvOutDTO getAllClient() throws CommonServiceException
	{
		try
		{
			Collection<ClientVO> clientVoCol = null;

			String query = "from ClientVO entitat where 1=1";

			clientVoCol = getClientDAO().find(query, null);

			return new ClientSrvOutDTO((Collection<ClientDTO>) getBeansConverter().convertCollection(clientVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	
	/**
	 * 
	 */
	public TipusClientSrvOutDTO getTipusClient(TipusClientSrvInDTO tipusClientSrvInDTO) throws CommonServiceException
	{
		try {
			TipusClientVO tipusClientVO = (TipusClientVO)getTipusClientDAO().load(tipusClientSrvInDTO.getTipusClientDTO().getId());
			return new TipusClientSrvOutDTO((TipusClientDTO)getBeansConverter().convert(tipusClientVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}
	
	/**
	 * 
	 */
	public TipusClientSrvOutDTO getAllTipusClient() throws CommonServiceException
	{
		try
		{
			Collection<TipusClientVO> tipusClientVoCol = null;

			String query = "from TipusClientVO entitat where 1=1";

			tipusClientVoCol = getTipusClientDAO().find(query, null);

			return new TipusClientSrvOutDTO((Collection<TipusClientDTO>) getBeansConverter().convertCollection(tipusClientVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	
	/**
	 * 
	 */
	public Integer getSemaforEstatIntervencions(String nifClient){
		return this.clientDAO.getSemaforEstatIntervencions(nifClient);
	}
	
	/**
	 * 
	 */
	public Integer getSemaforEstatFactures(String nifClient){
		return this.clientDAO.getSemaforEstatFactures(nifClient);		
	}

	/**
	 * @return the clientDAO
	 */
	public ClientDAO getClientDAO() {
		return clientDAO;
	}

	/**
	 * @param clientDAO the clientDAO to set
	 */
	public void setClientDAO(ClientDAO clientDAO) {
		this.clientDAO = clientDAO;
	}

	/**
	 * @return the tipusClientDAO
	 */
	public TipusClientDAO getTipusClientDAO() {
		return tipusClientDAO;
	}

	/**
	 * @param tipusClientDAO the tipusClientDAO to set
	 */
	public void setTipusClientDAO(TipusClientDAO tipusClientDAO) {
		this.tipusClientDAO = tipusClientDAO;
	}

}

