package com.ferri.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.beanutils.BeanUtils;

import com.ferri.persistence.bean.impl.ContracteVO;
import com.ferri.persistence.bean.impl.FacturaVO;
import com.ferri.persistence.bean.impl.IntervencioVO;
import com.ferri.persistence.bean.impl.TipusPeriodeVO;
import com.ferri.persistence.dao.interfaces.ContadorDAO;
import com.ferri.persistence.dao.interfaces.ContracteDAO;
import com.ferri.persistence.dao.interfaces.IntervencioDAO;
import com.ferri.persistence.dao.interfaces.TipusPeriodeDAO;
import com.ferri.portlet.utils.Format;
import com.ferri.service.commons.GenericBaseServiceImpl;
import com.ferri.service.dto.ContracteDTO;
import com.ferri.service.dto.FacturaDTO;
import com.ferri.service.dto.IntervencioDTO;
import com.ferri.service.dto.TipusPeriodeDTO;
import com.ferri.service.dto.input.ContracteSrvInDTO;
import com.ferri.service.dto.input.IntervencioSrvInDTO;
import com.ferri.service.dto.input.TipusPeriodeSrvInDTO;
import com.ferri.service.dto.output.ContracteSrvOutDTO;
import com.ferri.service.dto.output.FacturaSrvOutDTO;
import com.ferri.service.dto.output.IntervencioSrvOutDTO;
import com.ferri.service.dto.output.TipusPeriodeSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.ContracteService;

public class ContracteServiceImpl extends GenericBaseServiceImpl implements ContracteService {

	private ContracteDAO contracteDAO = null;
	private IntervencioDAO intervencioDAO = null;  
	private TipusPeriodeDAO tipusPeriodeDAO = null;
	private ContadorDAO contadorDAO = null; 
	
	/**
	 * 
	 */
	public Boolean updateContracte(ContracteSrvInDTO contracteSrvInDTO) throws CommonServiceException {
		try {
			ContracteVO tmpContracteVO = (ContracteVO)getBeansConverter().convert(contracteSrvInDTO.getContracteDTO());
			ContracteVO contracteVO = (ContracteVO) getContracteDAO().load(tmpContracteVO.getNumContracte());

			BeanUtils.copyProperties(contracteVO, tmpContracteVO);

			getContracteDAO().update(contracteVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String createContracte(ContracteSrvInDTO contracteSrvInDTO) throws CommonServiceException
	{
		try {
			ContracteDTO contracteDTO = contracteSrvInDTO.getContracteDTO();

			ContracteVO contracteVO = (ContracteVO)getBeansConverter().convert(contracteDTO);

			String id = (String) getContracteDAO().create(contracteVO);
			contracteDTO.setNumContracte(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deleteContracte(ContracteSrvInDTO contracteSrvInDTO) throws CommonServiceException
	{
		try {
			getContracteDAO().removeByPk(contracteSrvInDTO.getContracteDTO().getNumContracte());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public ContracteSrvOutDTO getContracte(ContracteSrvInDTO contracteSrvInDTO) throws CommonServiceException
	{
		try {
			ContracteVO contracteVO = (ContracteVO)getContracteDAO().load(contracteSrvInDTO.getContracteDTO().getNumContracte());
			return new ContracteSrvOutDTO((ContracteDTO)getBeansConverter().convert(contracteVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public ContracteSrvOutDTO getContractesByCriteria(ContracteSrvInDTO contracteSrvInDTO) throws CommonServiceException {
		
		try{			
			Collection<ContracteVO> contracteVoCol = null;

			StringBuffer query = new StringBuffer("from ContracteVO entitat where 1=1");
			if(contracteSrvInDTO !=null && contracteSrvInDTO.getContracteDTO() != null){				
				ContracteDTO dto = contracteSrvInDTO.getContracteDTO();
				
				if(dto.getNumContracte() != null && !dto.getNumContracte().trim().equals("")){
					query.append(" and upper(entitat.numContracte) = upper('").append(dto.getNumContracte()).append("')");
				}
				if(dto.getClient() != null && dto.getClient().getNif() != null && !dto.getClient().getNif().trim().equals("")){
					query.append(" and upper(entitat.client.nif) like upper('").append(dto.getClient().getNif()).append("%')");
				}
				
				if(dto.getTipusPeriode() != null && dto.getTipusPeriode().getId()!=null){
					query.append(" and entitat.tipusPeriode.id =").append(dto.getTipusPeriode().getId());
				}
				
				if(dto.getEstat() != null && dto.getEstat().getId() != null){
					query.append(" and entitat.estat.id = ").append(dto.getEstat().getId());
				}
				
				if(dto.getDataIni() != null){
					query.append(" and entitat.data >= ").append(Format.DATE_FORMAT_MYSQL.format(dto.getDataIni()));
				}
				if(dto.getDataFin() != null){					
					query.append(" and entitat.data <= ").append(Format.DATE_FORMAT_MYSQL.format(dto.getDataFin()));
				}
								
				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					query.append(" order by ").append(dto.getOrdre());
				}
			}
			contracteVoCol = getContracteDAO().find(query.toString(), null);

			return new ContracteSrvOutDTO((Collection<ContracteDTO>) getBeansConverter().convertCollection(contracteVoCol)); 
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public ContracteSrvOutDTO getAllContracte() throws CommonServiceException
	{
		try
		{
			Collection<ContracteVO> contracteVoCol = null;

			String query = "from ContracteVO entitat where 1=1";

			contracteVoCol = getContracteDAO().find(query, null);

			return new ContracteSrvOutDTO((Collection<ContracteDTO>) getBeansConverter().convertCollection(contracteVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	

	/**
	 * 
	 */
	public Boolean updateIntervencio(IntervencioSrvInDTO intervencioSrvInDTO) throws CommonServiceException {
		try {
			IntervencioVO tmpIntervencioVO = (IntervencioVO)getBeansConverter().convert(intervencioSrvInDTO.getIntervencioDTO());
			IntervencioVO intervencioVO = (IntervencioVO) getIntervencioDAO().load(tmpIntervencioVO.getId());

			BeanUtils.copyProperties(intervencioVO, tmpIntervencioVO);

			getIntervencioDAO().update(intervencioVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public Long createIntervencio(IntervencioSrvInDTO intervencioSrvInDTO) throws CommonServiceException
	{
		try {
			IntervencioDTO intervencioDTO = intervencioSrvInDTO.getIntervencioDTO();

			IntervencioVO intervencioVO = (IntervencioVO)getBeansConverter().convert(intervencioDTO);

			Long id = (Long) getIntervencioDAO().create(intervencioVO);
			intervencioDTO.setId(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deleteIntervencio(IntervencioSrvInDTO intervencioSrvInDTO) throws CommonServiceException
	{
		try {
			getIntervencioDAO().removeByPk(intervencioSrvInDTO.getIntervencioDTO().getId());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public IntervencioSrvOutDTO getIntervencio(IntervencioSrvInDTO intervencioSrvInDTO) throws CommonServiceException {
		try {
			IntervencioVO intervencioVO = (IntervencioVO)getIntervencioDAO().load(intervencioSrvInDTO.getIntervencioDTO().getId());
			return new IntervencioSrvOutDTO((IntervencioDTO)getBeansConverter().convert(intervencioVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public IntervencioSrvOutDTO getIntervenciosByCriteria(IntervencioSrvInDTO intervencioSrvInDTO) throws CommonServiceException{
		try{			
			Collection<IntervencioVO> intervencioVoCol = null;

			StringBuffer query = new StringBuffer("from IntervencioVO entitat where 1=1");
			if(intervencioSrvInDTO !=null && intervencioSrvInDTO.getIntervencioDTO() != null){				
				IntervencioDTO dto = intervencioSrvInDTO.getIntervencioDTO();

				if(dto.getContracte() != null && dto.getContracte().getNumContracte() != null && !dto.getContracte().getNumContracte().trim().equals("")){
					query.append(" and upper(entitat.contracte.numContracte) = upper('").append(dto.getContracte().getNumContracte()).append("')");
				}				
				if(dto.getClient() != null && dto.getClient().getNif() != null && !dto.getClient().getNif().trim().equals("")){
					query.append(" and upper(entitat.client.nif) like upper('").append(dto.getClient().getNif()).append("%')");
				}
				if(dto.getContracte() != null && dto.getContracte().getTipusPeriode() != null && dto.getContracte().getTipusPeriode().getId()!=null){
					query.append(" and entitat.contracte.tipusPeriode.id =").append(dto.getContracte().getTipusPeriode().getId());
				}
				if(dto.getEstat() != null && dto.getEstat().getId() != null){
					query.append(" and entitat.estat.id = ").append(dto.getEstat().getId());
				}

				if(dto.getMesAnyIni() != null){
					query.append("  and CONCAT(entitat.any , LPAD(entitat.mes, 2,'0')) >= ").append(Format.ANY_MES_FORMAT.format(dto.getMesAnyIni()));
					
				}
				if(dto.getMesAnyFin() != null){					
					query.append(" and CONCAT(entitat.any , LPAD(entitat.mes, 2,'0')) <= ").append(Format.ANY_MES_FORMAT.format(dto.getMesAnyFin()));
				}

				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					query.append(" order by ").append(dto.getOrdre());
				}
			} 
			intervencioVoCol = getIntervencioDAO().find(query.toString(), null);

			return new IntervencioSrvOutDTO((Collection<IntervencioDTO>) getBeansConverter().convertCollection(intervencioVoCol)); 
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public IntervencioSrvOutDTO getAllIntervencio() throws CommonServiceException
	{
		try
		{
			Collection<IntervencioVO> intervencioVoCol = null;

			String query = "from IntervencioVO entitat where 1=1";

			intervencioVoCol = getIntervencioDAO().find(query, null);

			return new IntervencioSrvOutDTO((Collection<IntervencioDTO>) getBeansConverter().convertCollection(intervencioVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	
	
	/**
	 * 
	 */
	public TipusPeriodeSrvOutDTO getTipusPeriode(TipusPeriodeSrvInDTO tipusPeriodeSrvInDTO) throws CommonServiceException
	{
		try {
			TipusPeriodeVO tipusPeriodeVO = (TipusPeriodeVO)getTipusPeriodeDAO().load(tipusPeriodeSrvInDTO.getTipusPeriodeDTO().getId());
			return new TipusPeriodeSrvOutDTO((TipusPeriodeDTO)getBeansConverter().convert(tipusPeriodeVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	
	/**
	 * 
	 */
	public TipusPeriodeSrvOutDTO getAllTipusPeriode() throws CommonServiceException
	{
		try
		{
			Collection<TipusPeriodeVO> tipusPeriodeVoCol = null;

			String query = "from TipusPeriodeVO entitat where 1=1";

			tipusPeriodeVoCol = getTipusPeriodeDAO().find(query, null);

			return new TipusPeriodeSrvOutDTO((Collection<TipusPeriodeDTO>) getBeansConverter().convertCollection(tipusPeriodeVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	
	/**
	 * 
	 */
	public String nextNumeroContracte() throws CommonServiceException
	{
		try
		{
			return getContadorDAO().nextNumeroContracte();

		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}	


	/**
	 * @return the intervencioDAO
	 */
	public ContracteDAO getContracteDAO() {
		return contracteDAO;
	}


	/**
	 * @param contracteDAO the contracteDAO to set
	 */
	public void setContracteDAO(ContracteDAO contracteDAO) {
		this.contracteDAO = contracteDAO;
	}


	/**
	 * @return the intervencioDAO
	 */
	public IntervencioDAO getIntervencioDAO() {
		return intervencioDAO;
	}


	/**
	 * @param intervencioDAO the intervencioDAO to set
	 */
	public void setIntervencioDAO(IntervencioDAO intervencioDAO) {
		this.intervencioDAO = intervencioDAO;
	}


	/**
	 * @return the tipusPeriodeDAO
	 */
	public TipusPeriodeDAO getTipusPeriodeDAO() {
		return tipusPeriodeDAO;
	}


	/**
	 * @param tipusPeriodeDAO the tipusPeriodeDAO to set
	 */
	public void setTipusPeriodeDAO(TipusPeriodeDAO tipusPeriodeDAO) {
		this.tipusPeriodeDAO = tipusPeriodeDAO;
	}


	/**
	 * @return the contadorDAO
	 */
	public ContadorDAO getContadorDAO() {
		return contadorDAO;
	}


	/**
	 * @param contadorDAO the contadorDAO to set
	 */
	public void setContadorDAO(ContadorDAO contadorDAO) {
		this.contadorDAO = contadorDAO;
	}
	
}

