package com.ferri.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.bcel.classfile.Constant;
import org.apache.commons.beanutils.BeanUtils;

import com.ferri.persistence.bean.impl.ConstantsVO;
import com.ferri.persistence.bean.impl.EstatVO;
import com.ferri.persistence.bean.impl.FacturaVO;
import com.ferri.persistence.bean.impl.PressupostVO;
import com.ferri.persistence.dao.interfaces.ConstantsDAO;
import com.ferri.persistence.dao.interfaces.ContadorDAO;
import com.ferri.persistence.dao.interfaces.EstatDAO;
import com.ferri.persistence.dao.interfaces.FacturaDAO;
import com.ferri.persistence.dao.interfaces.PressupostDAO;
import com.ferri.portlet.utils.Format;
import com.ferri.service.commons.GenericBaseServiceImpl;
import com.ferri.service.dto.ConstantsDTO;
import com.ferri.service.dto.EstatDTO;
import com.ferri.service.dto.FacturaDTO;
import com.ferri.service.dto.PressupostDTO;
import com.ferri.service.dto.input.ConstantsSrvInDTO;
import com.ferri.service.dto.input.EstatSrvInDTO;
import com.ferri.service.dto.input.FacturaSrvInDTO;
import com.ferri.service.dto.input.PressupostSrvInDTO;
import com.ferri.service.dto.output.ConstantsSrvOutDTO;
import com.ferri.service.dto.output.EstatSrvOutDTO;
import com.ferri.service.dto.output.FacturaSrvOutDTO;
import com.ferri.service.dto.output.PressupostSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.FacturaService;

public class FacturaServiceImpl extends GenericBaseServiceImpl implements FacturaService {

	private FacturaDAO facturaDAO = null;
	private PressupostDAO pressupostDAO = null;
	private EstatDAO estatDAO = null;
	private ContadorDAO contadorDAO = null;
	private ConstantsDAO constantsDAO = null;

	/**
	 * 
	 */
	public Boolean updateFactura(FacturaSrvInDTO facturaSrvInDTO) throws CommonServiceException {
		try {
			FacturaVO tmpFacturaVO = (FacturaVO)getBeansConverter().convert(facturaSrvInDTO.getFacturaDTO());
			FacturaVO facturaVO = (FacturaVO) getFacturaDAO().load(tmpFacturaVO.getNumFactura());

			BeanUtils.copyProperties(facturaVO, tmpFacturaVO);

			getFacturaDAO().update(facturaVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String createFactura(FacturaSrvInDTO facturaSrvInDTO) throws CommonServiceException
	{
		try {
			FacturaDTO facturaDTO = facturaSrvInDTO.getFacturaDTO();

			FacturaVO facturaVO = (FacturaVO)getBeansConverter().convert(facturaDTO);

			String id = (String) getFacturaDAO().create(facturaVO);
			facturaDTO.setNumFactura(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deleteFactura(FacturaSrvInDTO facturaSrvInDTO) throws CommonServiceException
	{
		try {
			getFacturaDAO().removeByPk(facturaSrvInDTO.getFacturaDTO().getNumFactura());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public FacturaSrvOutDTO getFactura(FacturaSrvInDTO facturaSrvInDTO) throws CommonServiceException
	{
		try {
			FacturaVO facturaVO = (FacturaVO)getFacturaDAO().load(facturaSrvInDTO.getFacturaDTO().getNumFactura());
			return new FacturaSrvOutDTO((FacturaDTO)getBeansConverter().convert(facturaVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public FacturaSrvOutDTO getFacturasByCriteria(FacturaSrvInDTO facturaSrvInDTO) throws CommonServiceException {
		
		try{			
			Collection<FacturaVO> facturaVoCol = null;

			StringBuffer query = new StringBuffer("from FacturaVO entitat where 1=1");
			if(facturaSrvInDTO !=null && facturaSrvInDTO.getFacturaDTO() != null){				
				FacturaDTO dto = facturaSrvInDTO.getFacturaDTO();
				
				if(dto.getNumFactura() != null && !dto.getNumFactura().trim().equals("")){
					query.append(" and upper(entitat.numFactura) = upper('").append(dto.getNumFactura()).append("')");
				}
				if(dto.getClient() != null && dto.getClient().getNif() != null && !dto.getClient().getNif().trim().equals("")){
					query.append(" and upper(entitat.client.nif) like upper('").append(dto.getClient().getNif()).append("%')");
				}
				
				if(dto.getConcepte() != null && !dto.getConcepte().trim().equals("")){
					query.append(" and upper(entitat.concepte) like upper('%").append(dto.getConcepte().trim()).append("%')");
				}
				
				if(dto.getEstat() != null && dto.getEstat().getId() != null){
					query.append(" and entitat.estat.id = ").append(dto.getEstat().getId());
				}
				
				if(dto.getDataIni() != null){
					query.append(" and entitat.data >= ").append(Format.DATE_FORMAT_MYSQL.format(dto.getDataIni()));
				}
				if(dto.getDataFin() != null){
					query.append(" and entitat.data <= ").append(Format.DATE_FORMAT_MYSQL.format(dto.getDataFin()));
				}

				if(dto.getBaseIni() != null){
					query.append(" and entitat.baseImponible >= ").append(dto.getBaseIni());
				}
				if(dto.getBaseFin() != null){
					query.append(" and entitat.baseImponible <= ").append(dto.getBaseFin());
				}
				
				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					/**
					 * TODO
					 */
					//Orden Tal cual:							
					//entitat.baseImponible, entitat.data
					//entitat.data, entitat.baseImponible
					
					//entitat.client.nom, entitat.data
					//entitat.client.nom, entitat.baseImponible
					query.append(" order by ").append(dto.getOrdre());
				}
			}
			facturaVoCol = getFacturaDAO().find(query.toString(), null);

			return new FacturaSrvOutDTO((Collection<FacturaDTO>) getBeansConverter().convertCollection(facturaVoCol)); 
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public FacturaSrvOutDTO getAllFactura() throws CommonServiceException
	{
		try
		{
			Collection<FacturaVO> facturaVoCol = null;

			String query = "from FacturaVO entitat where 1=1";

			facturaVoCol = getFacturaDAO().find(query, null);

			return new FacturaSrvOutDTO((Collection<FacturaDTO>) getBeansConverter().convertCollection(facturaVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	

	/**
	 * 
	 */
	public Boolean updatePressupost(PressupostSrvInDTO pressupostSrvInDTO) throws CommonServiceException {
		try {
			PressupostVO tmpPressupostVO = (PressupostVO)getBeansConverter().convert(pressupostSrvInDTO.getPressupostDTO());
			PressupostVO pressupostVO = (PressupostVO) getPressupostDAO().load(tmpPressupostVO.getNumPressupost());

			BeanUtils.copyProperties(pressupostVO, tmpPressupostVO);

			getPressupostDAO().update(pressupostVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String createPressupost(PressupostSrvInDTO pressupostSrvInDTO) throws CommonServiceException
	{
		try {
			PressupostDTO pressupostDTO = pressupostSrvInDTO.getPressupostDTO();

			PressupostVO pressupostVO = (PressupostVO)getBeansConverter().convert(pressupostDTO);

			String id = (String) getPressupostDAO().create(pressupostVO);
			pressupostDTO.setNumPressupost(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deletePressupost(PressupostSrvInDTO pressupostSrvInDTO) throws CommonServiceException
	{
		try {
			getPressupostDAO().removeByPk(pressupostSrvInDTO.getPressupostDTO().getNumPressupost());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public PressupostSrvOutDTO getPressupost(PressupostSrvInDTO pressupostSrvInDTO) throws CommonServiceException
	{
		try {
			PressupostVO pressupostVO = (PressupostVO)getPressupostDAO().load(pressupostSrvInDTO.getPressupostDTO().getNumPressupost());
			return new PressupostSrvOutDTO((PressupostDTO)getBeansConverter().convert(pressupostVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public PressupostSrvOutDTO getPressupostsByCriteria(PressupostSrvInDTO pressupostSrvInDTO) throws CommonServiceException {
		try{			
			Collection<PressupostVO> pressupostVoCol = null;

			StringBuffer query = new StringBuffer("from PressupostVO entitat where 1=1");
			if(pressupostSrvInDTO !=null && pressupostSrvInDTO.getPressupostDTO() != null){				
				PressupostDTO dto = pressupostSrvInDTO.getPressupostDTO();
				
				if(dto.getNumPressupost() != null && !dto.getNumPressupost().trim().equals("")){
					query.append(" and upper(entitat.numPressupost) = upper('").append(dto.getNumPressupost()).append("')");
				}
				if(dto.getClient() != null && dto.getClient().getNif() != null && !dto.getClient().getNif().trim().equals("")){
					query.append(" and upper(entitat.client.nif) like upper('").append(dto.getClient().getNif()).append("%')");
				}
				
				if(dto.getConcepte() != null && !dto.getConcepte().trim().equals("")){
					query.append(" and upper(entitat.concepte) like upper('%").append(dto.getConcepte().trim()).append("%')");
				}
				
				if(dto.getEstat() != null && dto.getEstat().getId() != null){
					query.append(" and entitat.estat.id = ").append(dto.getEstat().getId());
				}
				
				if(dto.getDataIni() != null){
					query.append(" and entitat.data >= ").append(Format.DATE_FORMAT_MYSQL.format(dto.getDataIni()));
				}
				if(dto.getDataFin() != null){
					query.append(" and entitat.data <= ").append(Format.DATE_FORMAT_MYSQL.format(dto.getDataFin()));
				}

				if(dto.getBaseIni() != null){
					query.append(" and entitat.baseImponible >= ").append(dto.getBaseIni());
				}
				if(dto.getBaseFin() != null){
					query.append(" and entitat.baseImponible <= ").append(dto.getBaseFin());
				}
				
				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					query.append(" order by ").append(dto.getOrdre());
				}
			}
			pressupostVoCol = getPressupostDAO().find(query.toString(), null);

			return new PressupostSrvOutDTO((Collection<PressupostDTO>) getBeansConverter().convertCollection(pressupostVoCol)); 
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public PressupostSrvOutDTO getAllPressupost() throws CommonServiceException
	{
		try
		{
			Collection<PressupostVO> pressupostVoCol = null;

			String query = "from PressupostVO entitat where 1=1";

			pressupostVoCol = getPressupostDAO().find(query, null);

			return new PressupostSrvOutDTO((Collection<PressupostDTO>) getBeansConverter().convertCollection(pressupostVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	
	
	/**
	 * 
	 */
	public EstatSrvOutDTO getEstat(EstatSrvInDTO estatSrvInDTO) throws CommonServiceException
	{
		try {
			EstatVO estatVO = (EstatVO)getEstatDAO().load(estatSrvInDTO.getEstatDTO().getId());
			return new EstatSrvOutDTO((EstatDTO)getBeansConverter().convert(estatVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	
	public EstatSrvOutDTO getEstatsFactura() throws CommonServiceException{
		return  getEstatsByPrefix(EstatVO.PREFIX_FACTURA);
	}
	public EstatSrvOutDTO getEstatsPressupost() throws CommonServiceException{
		return  getEstatsByPrefix(EstatVO.PREFIX_PRESSUPOST);
	}
	public EstatSrvOutDTO getEstatsContracte() throws CommonServiceException{
		return  getEstatsByPrefix(EstatVO.PREFIX_CONTRACTE);
	}
	public EstatSrvOutDTO getEstatsIntervencio() throws CommonServiceException{
		return  getEstatsByPrefix(EstatVO.PREFIX_INTERVENCIO);
	}
	
	/**
	 * Cerca un llista de entitats Estat depenent del prefix. La llista resultant pot ser:
	 * estats de Factura, estats de Pressupost, estats de Contracte i estats de Intervencio. 
	 * @param prefix
	 * @return resultat cerca
	 * @throws CommonServiceException
	 */
	private EstatSrvOutDTO getEstatsByPrefix(String prefix) throws CommonServiceException {
		
		try
		{
			Collection<EstatVO> estatVoCol = null;

			String query = "from EstatVO entitat where entitat.descCa like '" + prefix + "%'";

			estatVoCol = getEstatDAO().find(query, null);

			return new EstatSrvOutDTO((Collection<EstatDTO>) getBeansConverter().convertCollection(estatVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}


	/**
	 * 
	 */
	public String nextNumeroFactura() throws CommonServiceException
	{
		try
		{
			return getContadorDAO().nextNumeroFactura();

		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String nextNumeroPressupost() throws CommonServiceException
	{
		try
		{
			return getContadorDAO().nextNumeroPressupost();

		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public ConstantsSrvOutDTO getConstants(ConstantsSrvInDTO constantsSrvInDTO) throws CommonServiceException
	{
		try {
			ConstantsVO constantsVO = (ConstantsVO)getConstantsDAO().load(constantsSrvInDTO.getConstantsDTO().getId());
			return new ConstantsSrvOutDTO((ConstantsDTO)getBeansConverter().convert(constantsVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}
	
	/**
	 * @return the facturaDAO
	 */
	public FacturaDAO getFacturaDAO() {
		return facturaDAO;
	}


	/**
	 * @param facturaDAO the facturaDAO to set
	 */
	public void setFacturaDAO(FacturaDAO facturaDAO) {
		this.facturaDAO = facturaDAO;
	}


	/**
	 * @return the pressupostDAO
	 */
	public PressupostDAO getPressupostDAO() {
		return pressupostDAO;
	}


	/**
	 * @param pressupostDAO the pressupostDAO to set
	 */
	public void setPressupostDAO(PressupostDAO pressupostDAO) {
		this.pressupostDAO = pressupostDAO;
	}


	/**
	 * @return the estatDAO
	 */
	public EstatDAO getEstatDAO() {
		return estatDAO;
	}


	/**
	 * @param estatDAO the estatDAO to set
	 */
	public void setEstatDAO(EstatDAO estatDAO) {
		this.estatDAO = estatDAO;
	}


	/**
	 * @return the contadorDAO
	 */
	public ContadorDAO getContadorDAO() {
		return contadorDAO;
	}


	/**
	 * @param contadorDAO the contadorDAO to set
	 */
	public void setContadorDAO(ContadorDAO contadorDAO) {
		this.contadorDAO = contadorDAO;
	}


	/**
	 * @return the constantsDAO
	 */
	public ConstantsDAO getConstantsDAO() {
		return constantsDAO;
	}


	/**
	 * @param constantsDAO the constantsDAO to set
	 */
	public void setConstantsDAO(ConstantsDAO constantsDAO) {
		this.constantsDAO = constantsDAO;
	}
		
}

