package com.ferri.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.beanutils.BeanUtils;

import com.ferri.persistence.bean.impl.AdministradorVO;
import com.ferri.persistence.bean.impl.GestoriaVO;
import com.ferri.persistence.dao.interfaces.AdministradorDAO;
import com.ferri.persistence.dao.interfaces.GestoriaDAO;
import com.ferri.service.commons.GenericBaseServiceImpl;
import com.ferri.service.dto.AdministradorDTO;
import com.ferri.service.dto.GestoriaDTO;
import com.ferri.service.dto.input.AdministradorSrvInDTO;
import com.ferri.service.dto.input.GestoriaSrvInDTO;
import com.ferri.service.dto.output.AdministradorSrvOutDTO;
import com.ferri.service.dto.output.GestoriaSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.GestoriaService;

public class GestoriaServiceImpl extends GenericBaseServiceImpl implements GestoriaService {

	private GestoriaDAO gestoriaDAO = null;
	private AdministradorDAO administradorDAO = null;

	/**
	 * 
	 */
	public Boolean updateGestoria(GestoriaSrvInDTO gestoriaSrvInDTO) throws CommonServiceException {
		try {
			GestoriaVO tmpGestoriaVO = (GestoriaVO)getBeansConverter().convert(gestoriaSrvInDTO.getGestoriaDTO());
			GestoriaVO gestoriaVO = (GestoriaVO) getGestoriaDAO().load(tmpGestoriaVO.getNif());

			BeanUtils.copyProperties(gestoriaVO, tmpGestoriaVO);

			getGestoriaDAO().update(gestoriaVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String createGestoria(GestoriaSrvInDTO gestoriaSrvInDTO) throws CommonServiceException
	{
		try {
			GestoriaDTO gestoriaDTO = gestoriaSrvInDTO.getGestoriaDTO();

			GestoriaVO gestoriaVO = (GestoriaVO)getBeansConverter().convert(gestoriaDTO);

			String id = (String) getGestoriaDAO().create(gestoriaVO);
			gestoriaDTO.setNif(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deleteGestoria(GestoriaSrvInDTO gestoriaSrvInDTO) throws CommonServiceException
	{
		try {
			getGestoriaDAO().removeByPk(gestoriaSrvInDTO.getGestoriaDTO().getNif());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public GestoriaSrvOutDTO getGestoria(GestoriaSrvInDTO gestoriaSrvInDTO) throws CommonServiceException
	{
		try {
			GestoriaVO gestoriaVO = (GestoriaVO)getGestoriaDAO().load(gestoriaSrvInDTO.getGestoriaDTO().getNif());
			return new GestoriaSrvOutDTO((GestoriaDTO)getBeansConverter().convert(gestoriaVO)); 
			
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);				
			}
	}

	/**
	 * 
	 */
	public GestoriaSrvOutDTO getGestoriasByCriteria(GestoriaSrvInDTO gestoriaSrvInDTO) throws CommonServiceException {
		try{			
			
			Collection<GestoriaVO> gestoriaVoCol = null;
			
			StringBuffer query = new StringBuffer("from GestoriaVO entitat where 1=1");
			if(gestoriaSrvInDTO !=null && gestoriaSrvInDTO.getGestoriaDTO() != null){
				GestoriaDTO dto = gestoriaSrvInDTO.getGestoriaDTO();
				if(dto.getNif() != null && !dto.getNif().trim().equals("")){
					query.append(" and upper(entitat.nif) like upper('").append(dto.getNif()).append("%')");
				}
				if(dto.getNom() != null && !dto.getNom().trim().equals("")){
					query.append(" and upper(entitat.nom) like upper('%").append(dto.getNom()).append("%')");
				}
				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					query.append(" order by entitat.").append(dto.getOrdre());
				}
			}
			gestoriaVoCol = getGestoriaDAO().find(query.toString(), null);

			return new GestoriaSrvOutDTO((Collection<GestoriaDTO>) getBeansConverter().convertCollection(gestoriaVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public GestoriaSrvOutDTO getAllGestoria() throws CommonServiceException
	{
		try
		{			
			Collection<GestoriaVO> gestoriaVoCol = null;

			String query = "from GestoriaVO entitat where 1=1";

			gestoriaVoCol = getGestoriaDAO().find(query, null);

			return new GestoriaSrvOutDTO((Collection<GestoriaDTO>) getBeansConverter().convertCollection(gestoriaVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}
	

	/**
	 * 
	 */
	public Boolean updateAdministrador(AdministradorSrvInDTO administradorSrvInDTO) throws CommonServiceException {
		try {
			AdministradorVO tmpAdministradorVO = (AdministradorVO)getBeansConverter().convert(administradorSrvInDTO.getAdministradorDTO());
			AdministradorVO administradorVO = (AdministradorVO) getAdministradorDAO().load(tmpAdministradorVO.getNif());

			BeanUtils.copyProperties(administradorVO, tmpAdministradorVO);

			getAdministradorDAO().update(administradorVO);
			return Boolean.TRUE;
		
		} catch (RuntimeException e) {
			throw new CommonServiceException(e);
		} catch (IllegalAccessException e) {
			throw new CommonServiceException(e); 
		} catch (InvocationTargetException e) {
			throw new CommonServiceException(e);
		}
	}
	
	/**
	 * 
	 */
	public String createAdministrador(AdministradorSrvInDTO administradorSrvInDTO) throws CommonServiceException
	{
		try {
			AdministradorDTO administradorDTO = administradorSrvInDTO.getAdministradorDTO();

			AdministradorVO administradorVO = (AdministradorVO)getBeansConverter().convert(administradorDTO);

			String id = (String) getAdministradorDAO().create(administradorVO);
			administradorDTO.setNif(id);
			return id; 
			
		} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
			
	}

	/**
	 * 
	 */
	public Boolean deleteAdministrador(AdministradorSrvInDTO administradorSrvInDTO) throws CommonServiceException
	{
		try {
			getAdministradorDAO().removeByPk(administradorSrvInDTO.getAdministradorDTO().getNif());
			return Boolean.TRUE;
			
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}			
	}

	/**
	 * 
	 */
	public AdministradorSrvOutDTO getAdministrador(AdministradorSrvInDTO administradorSrvInDTO) throws CommonServiceException
	{
		try {
			AdministradorVO administradorVO = (AdministradorVO)getAdministradorDAO().load(administradorSrvInDTO.getAdministradorDTO().getNif());
			return new AdministradorSrvOutDTO((AdministradorDTO)getBeansConverter().convert(administradorVO)); 
			} catch (RuntimeException e) {
				if(e.getMessage().startsWith("No row with the given identifier exists")){
					return null;
				}
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public AdministradorSrvOutDTO getAdministradorsByCriteria(AdministradorSrvInDTO administradorSrvInDTO) throws CommonServiceException {
		try{			
			
			Collection<AdministradorVO> administradorVoCol = null;
			
			StringBuffer query = new StringBuffer("from AdministradorVO entitat where 1=1");
			if(administradorSrvInDTO !=null && administradorSrvInDTO.getAdministradorDTO() != null){
				AdministradorDTO dto = administradorSrvInDTO.getAdministradorDTO();
				if(dto.getNif() != null && !dto.getNif().trim().equals("")){
					query.append(" and upper(entitat.nif) like upper('").append(dto.getNif()).append("%')");
				}
				if(dto.getNom() != null && !dto.getNom().trim().equals("")){
					query.append(" and upper(entitat.nom) like upper('%").append(dto.getNom()).append("%')");
				}
				if(dto.getGestoria() != null && dto.getGestoria().getNif() != null && !dto.getGestoria().getNif().equals("")){
					query.append(" and upper(entitat.gestoria.nif) like upper('").append(dto.getGestoria().getNif()).append("%')");
				}
				if(dto.getOrdre() != null && !dto.getOrdre().trim().equals("")){
					query.append(" order by entitat.").append(dto.getOrdre());
				}
			}
			administradorVoCol = getAdministradorDAO().find(query.toString(), null);

			return new AdministradorSrvOutDTO((Collection<AdministradorDTO>) getBeansConverter().convertCollection(administradorVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
		}
		
	}
	
	/**
	 * 
	 */
	public AdministradorSrvOutDTO getAllAdministrador() throws CommonServiceException
	{
		try
		{
			Collection<AdministradorVO> administradorVoCol = null;

			String query = "from AdministradorVO entitat where 1=1";

			administradorVoCol = getAdministradorDAO().find(query, null);

			return new AdministradorSrvOutDTO((Collection<AdministradorDTO>) getBeansConverter().convertCollection(administradorVoCol)); 
			} catch (RuntimeException e) {
				throw new CommonServiceException(e);
			}
	}

	/**
	 * 
	 */
	public Integer countClients(String nif){
		return this.gestoriaDAO.countClients(nif);
	}
	
	public Integer countAdrministradors(String nif){
		return this.gestoriaDAO.countAdrministradors(nif);
	}
	
	public Integer countClientsAdministradors(String nif){
		return this.administradorDAO.countClients(nif);
	}

	/**
	 * @return the gestoriaDAO
	 */
	public GestoriaDAO getGestoriaDAO() {
		return gestoriaDAO;
	}


	/**
	 * @param gestoriaDAO the gestoriaDAO to set
	 */
	public void setGestoriaDAO(GestoriaDAO gestoriaDAO) {
		this.gestoriaDAO = gestoriaDAO;
	}


	/**
	 * @return the administradorDAO
	 */
	public AdministradorDAO getAdministradorDAO() {
		return administradorDAO;
	}


	/**
	 * @param administradorDAO the administradorDAO to set
	 */
	public void setAdministradorDAO(AdministradorDAO administradorDAO) {
		this.administradorDAO = administradorDAO;
	}
		
}

