package com.ferri.service.interfaces;

import com.ferri.service.dto.input.ClientSrvInDTO;
import com.ferri.service.dto.input.TipusClientSrvInDTO;
import com.ferri.service.dto.output.ClientSrvOutDTO;
import com.ferri.service.dto.output.TipusClientSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;

/**
 * Servei de la capa de negoci per les enitats Client i TipusClient
 * @author dferri
 *
 */
public abstract interface ClientService {
  //Client	

  /**
   * Actualitza una instancia Client
   * @param paramClientSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean updateClient(ClientSrvInDTO paramClientSrvInDTO) throws CommonServiceException;

  /**
   * Crea una instancia Client
   * @param paramClientSrvInDTO
   * @return String key
   * @throws CommonServiceException
   */
  public abstract String createClient(ClientSrvInDTO paramClientSrvInDTO) throws CommonServiceException;

  /**
   * Elimina una instancia Client
   * @param paramClientSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean deleteClient(ClientSrvInDTO paramClientSrvInDTO) throws CommonServiceException;
  
  /**
   * Busca sobre Clients amb els criteris informats
   * @param paramClientSrvInDTO
   * @return ClientSrvInDTO amb un Client
   * @throws CommonServiceException
   */
  public abstract ClientSrvOutDTO getClient(ClientSrvInDTO paramClientSrvInDTO) throws CommonServiceException;

  /**
   * Busca sobre Clients amb els criteris informats
   * @param paramClientSrvInDTO
   * @return ClientSrvInDTO amb una llista de Clients
   * @throws CommonServiceException
   */
  public ClientSrvOutDTO getClientsByCriteria(ClientSrvInDTO paramClientSrvInDTO) throws CommonServiceException;

  /**
   * Carrega tots els Clients
   * @return ClientSrvInDTO amb tots els Clients
   * @throws CommonServiceException
   */
  public abstract ClientSrvOutDTO getAllClient() throws CommonServiceException;
  

  //Extras Client
  
  /**
   * Cerca l'estat generic de les intervencions d'un Client
   * @param nifClient
   * @return Integer amb la constant de color
   */
  public Integer getSemaforEstatIntervencions(String nifClient);
  
  /**
   * Cerca l'estat generic de les factures d'un Client
   * @param nifClient
   * @return Integer amb la constants de color
   */
  public Integer getSemaforEstatFactures(String nifClient);
  
  
  //TipusClient
  
  /**
   * Busca sobre TipusClients amb els criteris informats
   * @param paramTipusClientSrvInDTO
   * @return TipusClientSrvInDTO amb un TipusClient
   * @throws CommonServiceException
   */
  public abstract TipusClientSrvOutDTO getTipusClient(TipusClientSrvInDTO paramTipusClientSrvInDTO) throws CommonServiceException;

  /**
   * Carrega tots els TipusClients
   * @return TipusClientSrvOutDTO amb tots els TipusClients
   * @throws CommonServiceException
   */
  public abstract TipusClientSrvOutDTO getAllTipusClient() throws CommonServiceException;
}

