package com.ferri.service.interfaces;

import com.ferri.service.dto.input.ContracteSrvInDTO;
import com.ferri.service.dto.input.IntervencioSrvInDTO;
import com.ferri.service.dto.input.TipusPeriodeSrvInDTO;
import com.ferri.service.dto.output.ContracteSrvOutDTO;
import com.ferri.service.dto.output.IntervencioSrvOutDTO;
import com.ferri.service.dto.output.TipusPeriodeSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;

/**
 * Servei de la capa de negoci per les enitats Contracte, Intervencio i Contador(numeraci de Contractes).
 * @author dferri
 *
 */
public abstract interface ContracteService {
  //Contracte

  /**
   * Actualitza una instancia Contracte
   * @param paramClientSrvInDTO
   * @return boolean resultat boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean updateContracte(ContracteSrvInDTO paramContracteSrvInDTO) throws CommonServiceException;

  /**
   * Crea una instancia Contracte
   * @param paramContracteSrvInDTO
   * @return String key
   * @throws CommonServiceException
   */
  public abstract String createContracte(ContracteSrvInDTO paramContracteSrvInDTO) throws CommonServiceException;

  /**
   * Elimina una instancia Contracte
   * @param paramContracteSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean deleteContracte(ContracteSrvInDTO paramContracteSrvInDTO) throws CommonServiceException;
  
  /**
   * Busca sobre Contractes amb els criteris informats
   * @param paramContracteSrvInDTO
   * @return ContracteSrvInDTO amb un Contracte
   * @throws CommonServiceException
   */
  public abstract ContracteSrvOutDTO getContracte(ContracteSrvInDTO paramContracteSrvInDTO) throws CommonServiceException;

  /**
   * Busca sobre Contractes amb els criteris informats
   * @param paramContracteSrvInDTO
   * @return ContracteSrvInDTO amb una llista de Contractes
   * @throws CommonServiceException
   */
  public ContracteSrvOutDTO getContractesByCriteria(ContracteSrvInDTO paramContracteSrvInDTO) throws CommonServiceException;

  /**
   * Carrega tots els Contractes
   * @return ContracteSrvInDTO amb tots els Contractes
   * @throws CommonServiceException
   */
  public abstract ContracteSrvOutDTO getAllContracte() throws CommonServiceException;
  
  //Intervencio
  
  /**
   * Actualitza una instancia Intervencio
   * @param paramClientSrvInDTO
   * @return boolean resultat boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean updateIntervencio(IntervencioSrvInDTO paramIntervencioSrvInDTO) throws CommonServiceException;

  /**
   * Crea una instancia Intervencio
   * @param paramIntervencioSrvInDTO
   * @return Log key
   * @throws CommonServiceException
   */
  public abstract Long createIntervencio(IntervencioSrvInDTO paramIntervencioSrvInDTO) throws CommonServiceException;

  /**
   * Elimina una instancia Intervencio
   * @param paramIntervencioSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean deleteIntervencio(IntervencioSrvInDTO paramIntervencioSrvInDTO) throws CommonServiceException;
  
  /**
   * Busca sobre Intervencions amb els criteris informats
   * @param paramIntervencioSrvInDTO
   * @return IntervencioSrvInDTO amb una Intercencio
   * @throws CommonServiceException
   */
  public abstract IntervencioSrvOutDTO getIntervencio(IntervencioSrvInDTO paramIntervencioSrvInDTO) throws CommonServiceException;

  /**
   * Busca sobre Intervencions amb els criteris informats
   * @param paramIntervencioSrvInDTO
   * @return IntervencioSrvInDTO amb una llista de Intervencions
   * @throws CommonServiceException
   */
  public IntervencioSrvOutDTO getIntervenciosByCriteria(IntervencioSrvInDTO paramIntervencioSrvInDTO) throws CommonServiceException;

  /**
   * Carrega totes les Intervencions
   * @return IntervencioSrvInDTO amb totes les Intervencions
   * @throws CommonServiceException
   */
  public abstract IntervencioSrvOutDTO getAllIntervencio() throws CommonServiceException;
  
  
  //TipusPeriode
  /**
   * Busca sobre TipusPeriode amb els criteris informats
   * @param paramTipusPeriodeSrvInDTO
   * @return TipusPeriodeSrvInDTO amb un TipusPeriode
   * @throws CommonServiceException
   */
  public abstract TipusPeriodeSrvOutDTO getTipusPeriode(TipusPeriodeSrvInDTO paramTipusPeriodeSrvInDTO) throws CommonServiceException;
  
  /**
   * Carrega tots els TipusPeriode
   * @return TipusPeriodeSrvInDTO amb tots els TipusPeriode
   * @throws CommonServiceException
   */
  public abstract TipusPeriodeSrvOutDTO getAllTipusPeriode() throws CommonServiceException;
  
  
  //Contador
  /**
   * Crida el segent numero de Contracte
   * @return String amb el segent numero
   * @throws CommonServiceException
   */
	public String nextNumeroContracte() throws CommonServiceException;
}

