package com.ferri.service.interfaces;

import com.ferri.service.dto.input.ConstantsSrvInDTO;
import com.ferri.service.dto.input.EstatSrvInDTO;
import com.ferri.service.dto.input.FacturaSrvInDTO;
import com.ferri.service.dto.input.PressupostSrvInDTO;
import com.ferri.service.dto.output.ConstantsSrvOutDTO;
import com.ferri.service.dto.output.EstatSrvOutDTO;
import com.ferri.service.dto.output.FacturaSrvOutDTO;
import com.ferri.service.dto.output.PressupostSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;

/**
 * Servei de la capa de negoci per les enitats Factura, Pressupost, Estat i Contador(numeraci de Factures i Pressupostos).
 * @author dferri
 *
 */
public abstract interface FacturaService {
	//Factura	

  /**
   * Actualitza una instancia Factura
   * @param paramFacturaSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean updateFactura(FacturaSrvInDTO paramFacturaSrvInDTO) throws CommonServiceException;

  /**
   * Crea una instancia Factura
   * @param paramFacturaSrvInDTO
   * @return String key
   * @throws CommonServiceException
   */
  public abstract String createFactura(FacturaSrvInDTO paramFacturaSrvInDTO) throws CommonServiceException;

  /**
   * Elimina una instancia Factura
   * @param paramFacturaSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean deleteFactura(FacturaSrvInDTO paramFacturaSrvInDTO) throws CommonServiceException;
  
  /**
   * Busca sobre Factures amb els criteris informats
   * @param paramFacturaSrvInDTO
   * @return FacturaSrvInDTO amb una Factura
   * @throws CommonServiceException
   */
  public abstract FacturaSrvOutDTO getFactura(FacturaSrvInDTO paramFacturaSrvInDTO) throws CommonServiceException;

  /**
   * Busca sobre Factures amb els criteris informats
   * @param paramFacturaSrvInDTO
   * @return FacturaSrvInDTO amb una llista de Factures
   * @throws CommonServiceException
   */
  public FacturaSrvOutDTO getFacturasByCriteria(FacturaSrvInDTO paramFacturaSrvInDTO) throws CommonServiceException;

  /**
   * Carrega totes les Factures
   * @return FacturaSrvInDTO amb totes les Factures
   * @throws CommonServiceException
   */
  public abstract FacturaSrvOutDTO getAllFactura() throws CommonServiceException;
  
  
  //Pressupost
  /**
   * Actualitza una instancia Pressupost
   * @param paramPressupostSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean updatePressupost(PressupostSrvInDTO paramPressupostSrvInDTO) throws CommonServiceException;

  /**
   * Crea una instancia Pressupost
   * @param paramPressupostSrvInDTO
   * @return String key
   * @throws CommonServiceException
   */
  public abstract String createPressupost(PressupostSrvInDTO paramPressupostSrvInDTO) throws CommonServiceException;

  /**
   * Elimina una instancia Pressupost
   * @param paramPressupostSrvInDTO
   * @return boolean resultat
   * @throws CommonServiceException
   */
  public abstract Boolean deletePressupost(PressupostSrvInDTO paramPressupostSrvInDTO) throws CommonServiceException;
  
  /**
   * Busca sobre Pressupostos amb els criteris informats
   * @param paramPressupostSrvInDTO
   * @return PressupostSrvInDTO amb un Pressupost
   * @throws CommonServiceException
   */
  public abstract PressupostSrvOutDTO getPressupost(PressupostSrvInDTO paramPressupostSrvInDTO) throws CommonServiceException;

  /**
   * Busca sobre Pressupostos amb els criteris informats
   * @param paramPressupostSrvInDTO
   * @return PressupostSrvInDTO amb una llista amb Pressupostos
   * @throws CommonServiceException
   */
  public PressupostSrvOutDTO getPressupostsByCriteria(PressupostSrvInDTO paramPressupostSrvInDTO) throws CommonServiceException;

  /**
   * Carrega tots els Pressupostos
   * @return PressupostSrvInDTO amb tots els pressupostos
   * @throws CommonServiceException
   */
  public abstract PressupostSrvOutDTO getAllPressupost() throws CommonServiceException;
  
  
  //Estat
  /**
   * Busca sobre Estats amb els criteris informats
   * @param paramEstatSrvInDTO
   * @return EstatSrvInDTO amb un Estat
   * @throws CommonServiceException
   */
  public abstract EstatSrvOutDTO getEstat(EstatSrvInDTO paramEstatSrvInDTO) throws CommonServiceException;

  /**
   * Carrega tots els Estats de Factura
   * @return EstatSrvInDTO amb tots els Estats de Factura
   * @throws CommonServiceException
   */
  public abstract EstatSrvOutDTO getEstatsFactura() throws CommonServiceException;
  
  /**
   * Carrega tots els Estats de Pressupost
   * @return EstatSrvInDTO amb tots els Estats de Pressupost
   * @throws CommonServiceException
   */
  public abstract EstatSrvOutDTO getEstatsPressupost() throws CommonServiceException;
  
  /**
   * Carrega tots els Estats de Contracte
   * @return EstatSrvInDTO amb tots els Estats de Contracte
   * @throws CommonServiceException
   */
  public abstract EstatSrvOutDTO getEstatsContracte() throws CommonServiceException;
  
  /**
   * Carrega tots els Estats de Intervencio
   * @return EstatSrvInDTO amb tots els Estats de Intervencio
   * @throws CommonServiceException
   */
  public abstract EstatSrvOutDTO getEstatsIntervencio() throws CommonServiceException;
  
  
  //Contador
  /**
   * Crida el segent numero de Factura
   * @return String amb el segent numero
   * @throws CommonServiceException
   */
  public String nextNumeroFactura() throws CommonServiceException;
	
  /**
   * Crida el segent numero de Pressupost
   * @return String amb el segent numero
   * @throws CommonServiceException
   */
  public String nextNumeroPressupost() throws CommonServiceException;
  
  
  //Constants
  /**
   * Busca sobre Constants amb els criteris informats
   * @param paramConstantsSrvInDTO
   * @return ConstantsSrvInDTO amb una Constant
   * @throws CommonServiceException
   */
  public abstract ConstantsSrvOutDTO getConstants(ConstantsSrvInDTO paramConstantsSrvInDTO) throws CommonServiceException;
  
}

