package com.ferri.service.interfaces;

import com.ferri.service.dto.input.AdministradorSrvInDTO;
import com.ferri.service.dto.input.GestoriaSrvInDTO;
import com.ferri.service.dto.output.AdministradorSrvOutDTO;
import com.ferri.service.dto.output.GestoriaSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;

/**
 * Servei de la capa de negoci per les enitats Gestoria i Administrador
 * @author dferri
 *
 */
public abstract interface GestoriaService{

	//Gestoria
	/**
	 * Actualitza una instancia Gestoria
	 * @param paramGestoriaSrvInDTO
	 * @return boolean resultat
	 * @throws CommonServiceException
	 */
	public abstract Boolean updateGestoria(GestoriaSrvInDTO paramGestoriaSrvInDTO) throws CommonServiceException;

	/**
	 * Crea una instancia Gestoria
	 * @param paramGestoriaSrvInDTO
	 * @return String key
	 * @throws CommonServiceException
	 */
	public abstract String createGestoria(GestoriaSrvInDTO paramGestoriaSrvInDTO) throws CommonServiceException;

	/**
	 * Elimina una instancia Gestoria
	 * @param paramGestoriaSrvInDTO
	 * @return boolean resultat
	 * @throws CommonServiceException
	 */
	public abstract Boolean deleteGestoria(GestoriaSrvInDTO paramGestoriaSrvInDTO) throws CommonServiceException;

	/**
	 * Busca sobre Gestoria amb els criteris informats
	 * @param paramGestoriaSrvInDTO
	 * @return GestoriaSrvInDTO amb una Gestoria
	 * @throws CommonServiceException
	 */
	public abstract GestoriaSrvOutDTO getGestoria(GestoriaSrvInDTO paramGestoriaSrvInDTO) throws CommonServiceException;

	/**
	 * Busca sobre Gestoria amb els criteris informats
	 * @param paramGestoriaSrvInDTO
	 * @return GestoriaSrvInDTO amb una llista amb Gestories
	 * @throws CommonServiceException
	 */
	public GestoriaSrvOutDTO getGestoriasByCriteria(GestoriaSrvInDTO paramGestoriaSrvInDTO) throws CommonServiceException;

	/**
	 * Carrega totes les Gestories
	 * @return GestoriaSrvInDTO amb totes les Gestories
	 * @throws CommonServiceException
	 */
	public abstract GestoriaSrvOutDTO getAllGestoria() throws CommonServiceException;

	
	//Extras contadores
	/**
	 * Compta els clients d'una gestoria amb aquesta clau
	 * @param nif
	 * @return Integer resultat
	 */
	public Integer countClients(String nif);

	/**
	 * Compta els administradors d'una gestoria amb aquesta clau
	 * @param nif
	 * @return Integer resultat
	 */
	public Integer countAdrministradors(String nif);


	//Administrador
	/**
	 * Actualitza una instancia Administrador
	 * @param paramAdministradorSrvInDTO
	 * @return boolean resultat
	 * @throws CommonServiceException
	 */
	public abstract Boolean updateAdministrador(AdministradorSrvInDTO paramAdministradorSrvInDTO) throws CommonServiceException;

	/**
	 * Crea una instancia Administrador
	 * @param paramAdministradorSrvInDTO
	 * @return String key
	 * @throws CommonServiceException
	 */
	public abstract String createAdministrador(AdministradorSrvInDTO paramAdministradorSrvInDTO) throws CommonServiceException;

	/**
	 * Elimina una instancia Administrador
	 * @param paramAdministradorSrvInDTO
	 * @return boolean resultat
	 * @throws CommonServiceException
	 */
	public abstract Boolean deleteAdministrador(AdministradorSrvInDTO paramAdministradorSrvInDTO) throws CommonServiceException;

	/**
	 * Busca sobre Administrador amb els criteris informats
	 * @param paramAdministradorSrvInDTO
	 * @return AdministradorSrvInDTO amb un Administrador
	 * @throws CommonServiceException
	 */
	public abstract AdministradorSrvOutDTO getAdministrador(AdministradorSrvInDTO paramAdministradorSrvInDTO) throws CommonServiceException;

	/**
	 * Busca sobre Administrador amb els criteris informats
	 * @param paramAdministradorSrvInDTO
	 * @return AdministradorSrvInDTO amb una llista de Administradors
	 * @throws CommonServiceException
	 */
	public AdministradorSrvOutDTO getAdministradorsByCriteria(AdministradorSrvInDTO paramAdministradorSrvInDTO) throws CommonServiceException;

	/**
	 * carrega tots els Administradors
	 * @return AdministradorSrvInDTO amb tots els Administradors
	 * @throws CommonServiceException
	 */
	public abstract AdministradorSrvOutDTO getAllAdministrador() throws CommonServiceException;

	//Extra contadores
	/**
	 * Compta els clients d'un administrador amb aquesta clau
	 * @param nif
	 * @return Integer resultat
	 */
	public Integer countClientsAdministradors(String nif);
}

