package com.ferri.servlet;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.AutowireCapableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.ferri.service.dto.AdministradorDTO;
import com.ferri.service.dto.GestoriaDTO;
import com.ferri.service.dto.input.AdministradorSrvInDTO;
import com.ferri.service.dto.output.AdministradorSrvOutDTO;
import com.ferri.service.exceptions.CommonServiceException;
import com.ferri.service.interfaces.GestoriaService;

/**
 * Servlet que escolta les crides jquery-ajax per carregar combos vinculats
 * @author dferri
 *
 */
public class ServletAjax extends HttpServlet implements ApplicationContextAware{

	private static final long serialVersionUID = 1L;

	private String[] locations = { 	"classpath:/com/ferri/config/persistenceContext.xml",
									"classpath:/com/ferri/config/dataSourcesContext.xml",
									"classpath:/com/ferri/config/serviceContext.xml" };	
	private ApplicationContext applicationContext = new ClassPathXmlApplicationContext(locations);
	
	private GestoriaService gestoriaService = null;
	
	/**
	 * Creador Intancia
	 */
	public ServletAjax(){
		applicationContext.getAutowireCapableBeanFactory().autowireBeanProperties(this, AutowireCapableBeanFactory.AUTOWIRE_BY_NAME, true);
	}
	
	
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		
		//Parametro Ajax:
		String id = req.getParameter("id");
		
		StringBuffer outSB = new StringBuffer();
		if(id != null && !id.equals("")){
			AdministradorSrvInDTO administradorSrvInDTO = new AdministradorSrvInDTO();
			AdministradorDTO administradorDTO = new AdministradorDTO();
			GestoriaDTO gestoriaDTO = new GestoriaDTO();
			gestoriaDTO.setNif(id);
			administradorDTO.setGestoria(gestoriaDTO);
			administradorSrvInDTO.setAdministradorDTO(administradorDTO);
			Collection<AdministradorDTO> collection = null;
			try {			
				AdministradorSrvOutDTO administradorSrvOutDTO = this.gestoriaService.getAdministradorsByCriteria(administradorSrvInDTO);
				collection = administradorSrvOutDTO.getAdministradorDTOList();
			} catch (CommonServiceException e) {
				throw new ServletException(e.getMessage(), e.getCause());
			}
			
			if(collection != null){
				for (AdministradorDTO dto : collection) {
					outSB.append("<option value='").append(dto.getNif()).append("'>").append(dto.getNom()).append("</option>");
				}
			}
		}
		
		resp.setContentType("text/html");
		OutputStream os = resp.getOutputStream();
		
		os.write(outSB.toString().getBytes());
		os.flush();
	}
	
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}


	/**
	 * @return the gestoriaService
	 */
	public GestoriaService getGestoriaService() {
		return gestoriaService;
	}


	/**
	 * @param gestoriaService the gestoriaService to set
	 */
	public void setGestoriaService(GestoriaService gestoriaService) {
		this.gestoriaService = gestoriaService;
	}
	
}
