package com.ferri.servlet;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.AutowireCapableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.ferri.persistence.bean.impl.ConstantsVO;
import com.ferri.portlet.utils.ExcelUtils;
import com.ferri.service.dto.ConstantsDTO;
import com.ferri.service.dto.FacturaDTO;
import com.ferri.service.dto.input.ConstantsSrvInDTO;
import com.ferri.service.interfaces.FacturaService;

/**
 * Servlet que escolta les crides per generar documents excel de Factures, possant-los al OutputStream de la response.
 * @author dferri
 *
 */
public class ServletExcel extends HttpServlet implements ApplicationContextAware{

	private static final long serialVersionUID = 1L;

	private String[] locations = { 	"classpath:/com/ferri/config/persistenceContext.xml",
									"classpath:/com/ferri/config/dataSourcesContext.xml",
									"classpath:/com/ferri/config/serviceContext.xml" };	
	private ApplicationContext applicationContext = new ClassPathXmlApplicationContext(locations);
	
	private FacturaService facturaService = null;
	
	public ServletExcel(){
		applicationContext.getAutowireCapableBeanFactory().autowireBeanProperties(this, AutowireCapableBeanFactory.AUTOWIRE_BY_NAME, true);
	}
	
	
	
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)  throws ServletException, IOException {
		
		try{
		Collection<FacturaDTO> facturaDTOList = (Collection<FacturaDTO>) req.getSession().getAttribute("llistaCercaExcel");
		
		//Antes de nada recuperamos los tipos actuales de IVA e IRPFdefinidos en constants
		ConstantsSrvInDTO constantsSrvInDTO = new ConstantsSrvInDTO();
		ConstantsDTO constantsDTO = new ConstantsDTO();
		
		constantsDTO.setId(ConstantsVO.KEY_IVA);
		constantsSrvInDTO.setConstantsDTO(constantsDTO);		
		Double percentIVA = this.facturaService.getConstants(constantsSrvInDTO).getConstantsDTO().getValor();
		
		constantsDTO.setId(ConstantsVO.KEY_IRPF);
		constantsSrvInDTO.setConstantsDTO(constantsDTO);
		Double percentIRPF = this.facturaService.getConstants(constantsSrvInDTO).getConstantsDTO().getValor();
		
		resp.setContentType("application/vnd.ms-excel");
		resp.setHeader("Content-Disposition", "attachment; filename=factures.xls");

		OutputStream os= resp.getOutputStream();
		ExcelUtils.generaExcelFactura(facturaDTOList, percentIVA, percentIRPF, req.getLocale(), os);
		
		}catch(Exception e){
			resp.setContentType("text/html");
			OutputStream os = resp.getOutputStream();
			String outString = "<html><body><h1>ERROR AL GENERAR EL EXCEL</h1></body></html>";
			os.write(outString.getBytes());
			os.flush();
		}
	}
	
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}


	/**
	 * @return the facturaService
	 */
	public FacturaService getFacturaService() {
		return facturaService;
	}

	/**
	 * @param facturaService the facturaService to set
	 */
	public void setFacturaService(FacturaService facturaService) {
		this.facturaService = facturaService;
	}
	
}
