package com.ferri.servlet;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.AutowireCapableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.ferri.persistence.bean.impl.ConstantsVO;
import com.ferri.portlet.utils.PdfUtils;
import com.ferri.service.dto.ConstantsDTO;
import com.ferri.service.dto.ContracteDTO;
import com.ferri.service.dto.FacturaDTO;
import com.ferri.service.dto.PressupostDTO;
import com.ferri.service.dto.input.ConstantsSrvInDTO;
import com.ferri.service.dto.input.ContracteSrvInDTO;
import com.ferri.service.dto.input.FacturaSrvInDTO;
import com.ferri.service.dto.input.PressupostSrvInDTO;
import com.ferri.service.dto.output.ContracteSrvOutDTO;
import com.ferri.service.dto.output.FacturaSrvOutDTO;
import com.ferri.service.dto.output.PressupostSrvOutDTO;
import com.ferri.service.interfaces.ContracteService;
import com.ferri.service.interfaces.FacturaService;

/**
 * Servlet que escolta les crides per generar documents pdf de Factures, Pressupostos o Contractes, possant-los al OutputStream de la response.
 * @author dferri
 *
 */
public class ServletPdf extends HttpServlet implements ApplicationContextAware{

	private static final long serialVersionUID = 1L;

	private String[] locations = { 	"classpath:/com/ferri/config/persistenceContext.xml",
									"classpath:/com/ferri/config/dataSourcesContext.xml",
									"classpath:/com/ferri/config/serviceContext.xml" };	
	private ApplicationContext applicationContext = new ClassPathXmlApplicationContext(locations);
	
	private FacturaService facturaService = null;
	private ContracteService contracteService = null;
	
	public ServletPdf(){
		applicationContext.getAutowireCapableBeanFactory().autowireBeanProperties(this, AutowireCapableBeanFactory.AUTOWIRE_BY_NAME, true);
	}
	
	
	
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)  throws ServletException, IOException {
		
		try{
		String id	 = req.getParameter("id");
		
		//De momento lo dejamos deprecado si hay tiempo busco una mejor manera.
		String xslPath = req.getRealPath("xsl");
		
		String xml = "";
		String xslname = "";
		
		//Antes de nada recuperamos los tipos actuales de IVA e IRPFdefinidos en constants
		ConstantsSrvInDTO constantsSrvInDTO = new ConstantsSrvInDTO();
		ConstantsDTO constantsDTO = new ConstantsDTO();
		
		constantsDTO.setId(ConstantsVO.KEY_IVA);
		constantsSrvInDTO.setConstantsDTO(constantsDTO);		
		Double percentIVA = this.facturaService.getConstants(constantsSrvInDTO).getConstantsDTO().getValor();
		
		constantsDTO.setId(ConstantsVO.KEY_IRPF);
		constantsSrvInDTO.setConstantsDTO(constantsDTO);
		Double percentIRPF = this.facturaService.getConstants(constantsSrvInDTO).getConstantsDTO().getValor();
		
		if(id.startsWith(PdfUtils.PDF_TIPUS_FACTURA)){
			FacturaSrvInDTO facturaSrvInDTO = new FacturaSrvInDTO();
			FacturaDTO facturaDTO = new FacturaDTO();
			facturaDTO.setNumFactura(id);
			facturaSrvInDTO.setFacturaDTO(facturaDTO);
			FacturaSrvOutDTO facturaSrvOutDTO = this.facturaService.getFactura(facturaSrvInDTO);
			facturaDTO = facturaSrvOutDTO.getFacturaDTO();
			
			xml	= facturaDTO.getXmlFop(xslPath, percentIVA, percentIRPF);	
			xslname = PdfUtils.XSL_FACTURA;
			
		} else if(id.startsWith(PdfUtils.PDF_TIPUS_PRESSUPOST)){
			PressupostSrvInDTO pressupostSrvInDTO = new PressupostSrvInDTO();
			PressupostDTO pressupostDTO = new PressupostDTO();
			pressupostDTO.setNumPressupost(id);
			pressupostSrvInDTO.setPressupostDTO(pressupostDTO);
			PressupostSrvOutDTO pressupostSrvOutDTO = this.facturaService.getPressupost(pressupostSrvInDTO);
			pressupostDTO = pressupostSrvOutDTO.getPressupostDTO();
			
			xml	= pressupostDTO.getXmlFop(xslPath, percentIVA, percentIRPF);
			xslname = PdfUtils.XSL_PRESSUPOST;
			
		} if(id.startsWith(PdfUtils.PDF_TIPUS_CONTRACTE)){
			ContracteSrvInDTO contracteSrvInDTO = new ContracteSrvInDTO();
			ContracteDTO contracteDTO = new ContracteDTO();
			contracteDTO.setNumContracte(id);
			contracteSrvInDTO.setContracteDTO(contracteDTO);
			ContracteSrvOutDTO contracteSrvOutDTO = this.contracteService.getContracte(contracteSrvInDTO);
			contracteDTO = contracteSrvOutDTO.getContracteDTO();
			
			xml = contracteDTO.getXmlFop(xslPath);			
			xslname = PdfUtils.XSL_CONTRACTE;
		}
		
		OutputStream os= resp.getOutputStream();
		PdfUtils.generatePdf(xml, xslPath + File.separator + xslname, os);
		resp.setContentType("application/pdf");
		
		}catch(Exception e){
			resp.setContentType("text/html");
			OutputStream os = resp.getOutputStream();
			String outString = "<html><body><h1>ERROR AL GENERAR EL PDF</h1></body></html>";
			os.write(outString.getBytes());
			os.flush();
		}
	}
	
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}


	/**
	 * @return the facturaService
	 */
	public FacturaService getFacturaService() {
		return facturaService;
	}

	/**
	 * @param facturaService the facturaService to set
	 */
	public void setFacturaService(FacturaService facturaService) {
		this.facturaService = facturaService;
	}

	/**
	 * @return the contracteService
	 */
	public ContracteService getContracteService() {
		return contracteService;
	}

	/**
	 * @param contracteService the contracteService to set
	 */
	public void setContracteService(ContracteService contracteService) {
		this.contracteService = contracteService;
	}
	
}
